/*
	$Id: Collectable.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/Collectable.hpp>
#include <librfc/Collection.hpp>

#include <stdlib.h>

USE_RFC_NAMESPACE

Collectable::Collectable( const String& strKey )
			: _strKey( strKey ),
			_lDeletionPolicy( 0 ),
			_bIsBeingDeleted( 0 )
{
	_pNext = NULL;
	_pPrevious = NULL;
	_pParent = NULL;
}

Collectable::Collectable( long lDeletionPolicy, const String& strKey )
			: _strKey( strKey ),
			_lDeletionPolicy( lDeletionPolicy ),
			_bIsBeingDeleted( 0 )
{
	_pNext = NULL;
	_pPrevious = NULL;
	_pParent = NULL;
}

Collectable::~Collectable()
{
	_bIsBeingDeleted = 1;

	if( _pParent != NULL )
	{
		_pParent->remove( this );
	}
}

const String& Collectable::key() const
{
	return _strKey;
}

void Collectable::remove( long bDelete )
{
	if( _pParent != NULL )
	{
		_pParent->remove( this, bDelete );
	}
}

void Collectable::addToCollectionHead( Collection* pParent )
{
	if( _pParent != NULL )
		throw CollectionException( CE_ALREADYMEMBER );

	_pParent = pParent;
	_pNext = _pParent->first();
	if( _pNext != NULL )
	{
		_pNext->_pPrevious = this;
	}
	_pParent->first( this );
	if( _pParent->last() == NULL )
	{
		_pParent->last( this );
	}
}

void Collectable::addToCollectionTail( Collection* pParent )
{
	if( _pParent != NULL )
		throw CollectionException( CE_ALREADYMEMBER );

	_pParent = pParent;
	_pPrevious = _pParent->last();
	if( _pPrevious != NULL )
	{
		_pPrevious->_pNext = this;
	}
	_pParent->last( this );
	if( _pParent->first() == NULL )
	{
		_pParent->first( this );
	}
}

Collectable* Collectable::next( Collection* pParent )
{
	if( _pParent != pParent )
		throw CollectionException( CE_NOTAMEMBER );

	return _pNext;
}

Collectable* Collectable::previous( Collection* pParent )
{
	if( _pParent != pParent )
		throw CollectionException( CE_NOTAMEMBER );

	return _pPrevious;
}

void Collectable::removeFrom( Collection* pParent )
{
	if( _pParent != pParent )
		throw CollectionException( CE_NOTAMEMBER );

	if( _pPrevious != NULL )
	{
		_pPrevious->_pNext = _pNext;
	}
	else
	{
		_pParent->first( _pNext );
	}

	if( _pNext != NULL )
	{
		_pNext->_pPrevious = _pPrevious;
	}
	else
	{
		_pParent->last( _pPrevious );
	}

	_pParent = NULL;
}

long Collectable::canIDeleteYou() const
{
	if( _bIsBeingDeleted )
		return 0;

	if( _lDeletionPolicy & COLDEL_COLLECTIONCANDELETE )
		return 1;

	return 0;
}
