/*
	$Id: Collectable.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __COLLECTABLE_HPP__
#define __COLLECTABLE_HPP__

#include <librfc/String.hpp>

#include <librfc/export.h>

// Deletion Policies
#define COLDEL_COLLECTIONCANDELETE	1	// If set, a collection can delete a collectable
#define COLDEL_REMOVALCASCADE		3	// If set, if a collectable is a member of multiple
										//  collections when it is removed from a deleting
										//  collection, it will remove itself from all other
										//  collections and permit itself to be deleted.

#include <librfc/CollectionException.hpp>

BEGIN_RFC_NAMESPACE
class Collection;

class RFC_Export Collectable
{
protected:
	Collectable* _pNext;
	Collectable* _pPrevious;
	Collection* _pParent;
	String _strKey;
	long _lDeletionPolicy;
	long _bIsBeingDeleted;

	virtual void addToCollectionHead( Collection* pParent );
	virtual void addToCollectionTail( Collection* pParent );
	virtual Collectable* next( Collection* pParent );
	virtual Collectable* previous( Collection* pParent );
	virtual void removeFrom( Collection* pParent );
	virtual long canIDeleteYou() const;
	long isBeingDeleted() const { return _bIsBeingDeleted; }

public:
	Collectable( const String& strKey = String() );
	Collectable( long lDeletionPolicy, const String& strKey = String() );
	virtual ~Collectable();

	const String& key() const;

	// Self-Removal of a Collectable from a collection relies on the remove member
	//  of the collection. Collectable::remove calls Collection::remove which calls
	//  the helper function Collectable::removeFrom( Collection* ) to actually
	//  perform the removal of the object from the collection
	virtual void remove( long bDelete = 0 );

	friend class Collection;
	friend class MultiCollectable;
};
END_RFC_NAMESPACE

#endif /* !__COLLECTABLE_HPP__ */

