/*
	$Id: Collection.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __COLLECTION_HPP__
#define __COLLECTION_HPP__

#include <librfc/Collectable.hpp>
#include <librfc/export.h>

#include <stdlib.h>

#if defined (_WINDOWS)
#	include <windows.h>
#	include <process.h>
#elif defined (LINUX)
#	include <pthread.h>
#endif

BEGIN_RFC_NAMESPACE
class RFC_Export Collection : public Collectable
{
private:
	Collectable* _pFirst;
	Collectable* _pLast;
	long _lDeleteMembersOnDestuction;
	unsigned long _ulMembers;
#if defined (_WINDOWS)
	CRITICAL_SECTION _cs;
#elif defined (LINUX)
	pthread_mutex_t* _pmx;
#endif

protected:
	void first( Collectable* pFirst ){ _pFirst = pFirst; }
	void last( Collectable* pLast ){ _pLast = pLast; }
	Collectable* first() const { return _pFirst; }
	Collectable* last() const { return _pLast; }

public:
	Collection( long lDeleteMembersOnDestuction = 0 );
	virtual ~Collection();

	virtual void addTail( Collectable* pitem );
	virtual void addHead( Collectable* pitem );
	virtual Collectable* next( Collectable* pCurrent );
	virtual Collectable* previous( Collectable* pCurrent );
	virtual Collectable* find( const String& strKey, Collectable* pCurrent = NULL );
	virtual void remove( Collectable* pRemove, long bDelete = 0 );
	virtual void removeAll( long bDelete = 0 );

	unsigned long count() const;

	friend class Collectable;
	friend class MultiCollectable;
};
END_RFC_NAMESPACE

#endif /* !__COLLECTION_HPP__ */
