/*
	$Id: DB.cpp 1270 2018-12-30 14:29:29Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/DB.hpp>
#include <librfc/DBPoolException.hpp>

#include <librfc/Socket.hpp>


USE_RFC_NAMESPACE

DB::DB( DBPoolMgr* pmgr )
	:_pdbP( NULL ),
	_pdbA( NULL ),
	_pmgrP( pmgr ),
	_pmgrA( NULL )
{
	_pdbP = _pmgrP->getPooledDB();
	if( _pdbP == NULL )
	{
		DBPoolException e( DBPE_EMPTY );
		throw e;
	}
}

DB::DB( DBAutoMgr* pmgr )
	:_pdbP( NULL ),
	_pdbA( NULL ),
	_pmgrP( NULL ),
	_pmgrA( pmgr )
{
	_pdbA = _pmgrA->getAutoDB();
	if( _pdbA == NULL )
	{
		DBPoolException e( DBPE_EMPTY );
		throw e;
	}
}

DB::~DB()
{
	if( _pdbP != NULL )
	{
		if( _pdbP->_db._bInTransaction )
			_pdbP->_db.rollback();

		_pmgrP->freePooledDB( _pdbP );
	}
	else if( _pdbA != NULL )
	{
		if( _pdbA->_db._bInTransaction )
			_pdbA->_db.rollback();

		_pmgrA->freeAutoDB( _pdbA );
	}
}

Database* DB::db()
{
	if( _pdbP != NULL )
		return &_pdbP->_db;
	else if( _pdbA != NULL )
		return &_pdbA->_db;
	else
		return NULL;
}

void DB::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl;
}

void DB::dump( TAG_RFC_NAMESPACE Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n";
}
