/*
	$Id: DBPoolMgr.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DBPOOLMGR_HPP__
#define __DBPOOLMGR_HPP__

#include <librfc/PooledDB.hpp>
#include <librfc/RFCBase.hpp>
#include <librfc/export.h>

#ifdef LINUX
#	include <pthread.h>
#endif

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export DBPoolMgr : public RFCBase
{
private:
#ifdef _DEBUG
	long _lTotal;
	long _lInUse;
#endif
	unsigned long _ulFailureCount;
	PooledDB* _pdbFirst;
	PooledDB* _pdbLast;
#if defined (_WINDOWS)
	CRITICAL_SECTION _cs;
#elif defined (LINUX)
	pthread_mutex_t _mx;
#endif
	unsigned long _ulStart;
	String _strDSN;
	String _strUserId;
	String _strPassword;
	long _lInitialSize;
	long _lMaxSize;
	long _lCurrentSize;
	bool _bResizeable;
	long addPooledDB( PooledDB* pdb );

public:
	DBPoolMgr();
	virtual ~DBPoolMgr();
	long setResizeable( const String& strSource, const String& strUserId, const String& strPassword,
						long lInitialSize = 0, long lMaxSize = -1 );
	long addPooledDB( const char* pszSource, const char* pszUserId, const char* pszPassword );
	PooledDB* getPooledDB();
	void freePooledDB( PooledDB* pdbFree );
	void statistics( char* pszStats );
	void refresh();

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__DBPOOLMGR_HPP__ */
