/*
	$Id: Database.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DATABASE_HPP__
#define __DATABASE_HPP__

#include <librfc/ODBCEnvironment.hpp>
#include <librfc/String.hpp>
#include <librfc/RFCBase.hpp>

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export Database : public RFCBase
{
private:
	long		_bConnValid;
	long		_bIsOpen;
	long		_bShouldBeAutocommit;
	long		_bInTransaction;
	long		_lRowCount;

	char		_szSource[32];
	char		_szUser[32];
	char		_szAuth[32];

	String		_strLastError;

	SQLHANDLE	_hConn;

public:
	Database();
	virtual ~Database();

	SQLHANDLE handle();
	long open( const char* pszSource, const char* pszUser, const char* pszAuth );
	long close();
	long isOpen(){ return _bIsOpen; }
	String lastError(){ return _strLastError; }
	long execute( const char* pszStatement );
	long enableAutoCommit( long lEnable = 1 );
	void begin();
	long commit();
	long rollback();
	long rowCount() const;

	static char* DeQuote( char* pszTgt, const char* pszSrc );

	friend class ResultSet;
	friend class DB;

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__DATABASE_HPP__ */
