/*
	$Id: Date.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __DATE_HPP__
#define __DATE_HPP__

#include <librfc/String.hpp>
#include <librfc/FCConfig.hpp>
#include <librfc/export.h>
#include <librfc/RFCBase.hpp>

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export Date : public RFCBase
{
public:
	typedef enum { sunday, monday, tuesday, wednesday, thursday, friday, saturday } edayOfWeek;
protected:
	long _lYear;
	long _lMonth;
	long _lDay;
	String _strFormat;

public:
	// __ctors
	Date();
	Date( long lYear, long lMonth, long lDay );
	Date( const String& strDate, const String& strFormat = GetFCCString( SE_DEFAULT_DATE_FORMAT ) );
	Date( const Date& date );
	virtual ~Date();

	// operators
	const Date& operator= ( const String& strDate );
	const Date& operator= ( const Date& date );

	// accessors
	long year() const { return _lYear; }
	long month() const { return _lMonth; }
	long day() const { return _lDay; }

	edayOfWeek dayOfWeek() const;
	long dayOfYear() const;

	long lastDay() const;

	// mutators
	void year( long lYear );
	void month( long lMonth );
	void day( long lDay );

	String format( const String& strFormat = GetFCCString( SE_DEFAULT_DATE_FORMAT ) ) const;

	static String shortMonth( long lIndex );
	static String longMonth( long lIndex );

	static Date now();

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__DATE_HPP__ */
