/*
	$Id: Directory.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Directory_hpp__
#define __Directory_hpp__

#include <librfc/String.hpp>

#if defined (LINUX)
#include <fcntl.h>
#include <sys/stat.h>
#include <regex.h>
#include <dirent.h>
#elif defined (_WINDOWS)
#endif


BEGIN_RFC_NAMESPACE
class Directory
{
private:
	String _strPath;
	String _strPattern;
	bool _bOpen;
#if defined (LINUX)
	static regex_t _re;
#endif
	long _lCount;

public:
	Directory();
	virtual ~Directory();

	long open( const String& strPath, const String& strPattern );
	long next();
	String path() const;
	String filename() const;
	String pathname() const;
	long forEach( long(*fn)( const Directory&, void* ) );

	static long processThis( const Directory& directory, void* pvContext );

#if defined (LINUX)
	static int filter( const dirent* pentry );
#endif /* LINUX */
};
END_RFC_NAMESPACE

#endif /* !__Directory_hpp__ */
