/*
	$Id: MultiCollectable.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/MultiCollectable.hpp>


USE_RFC_NAMESPACE

MultiCollectable::Position::Position()
				: Collectable(),
				_pcollection( NULL )
{
}

MultiCollectable::Position::~Position()
{
}

MultiCollectable::MultiCollectable( const String& strKey )
				: Collectable( strKey )
{
}

MultiCollectable::MultiCollectable( long lDeletionPolicy, const String& strKey )
				: Collectable( lDeletionPolicy, strKey )
{
}

MultiCollectable::~MultiCollectable()
{
	_bIsBeingDeleted = 1;

	// Iterate through collection of parents, removing from each one
	Position* pposition = NULL;
	while( ( pposition = (Position*)_collectionS.next( pposition ) ) != NULL )
	{
		pposition->_pcollection->remove( this );
	}
}

//Collection* parent() const { return _pParent; }

void MultiCollectable::addToCollectionHead( Collection* pcollection )
{
	// Firstly, we iterate through the Positions to make sure that the object is
	//  not already a member of this collection.
	Position* pposition = NULL;
	while( ( pposition = (Position*)_collectionS.next( pposition ) ) != NULL )
	{
		if( pposition->_pcollection == pcollection )
			throw CollectionException( CE_ALREADYMEMBER );
	}

	pposition = new Position;
	pposition->_pcollection = pcollection;
	pposition->_pNext = pposition->_pParent->first();
	if( pposition->_pNext != NULL )
	{
		pposition->_pcollection->_pNext->_pPrevious = this;
	}
	pposition->_pcollection->first( this );
	if( pposition->_pcollection->last() == NULL )
	{
		pposition->_pcollection->last( this );
	}
	_collectionS.addTail( pposition );
}

void MultiCollectable::addToCollectionTail( Collection* pcollection )
{
	// Firstly, we iterate through the Positions to make sure that the object is
	//  not already a member of this collection.
	Position* pposition = NULL;
	while( ( pposition = (Position*)_collectionS.next( pposition ) ) != NULL )
	{
		if( pposition->_pcollection == pcollection )
			throw CollectionException( CE_ALREADYMEMBER );
	}

	pposition = new Position;
	pposition->_pcollection = pcollection;
	pposition->_pPrevious = pposition->_pcollection->last();
	if( pposition->_pPrevious != NULL )
	{
		pposition->_pcollection->_pPrevious->_pNext = this;
	}
	pposition->_pcollection->last( this );
	if( pposition->_pcollection->first() == NULL )
	{
		pposition->_pcollection->first( this );
	}
	_collectionS.addTail( pposition );
}

Collectable* MultiCollectable::next( Collection* pcollection )
{
	// Firstly, we iterate through the Positions to make sure that the object is
	//  a member of this collection.
	Position* pposition = NULL;
	Position* ppositionFound = NULL;
	while( ( pposition = (Position*)_collectionS.next( pposition ) ) != NULL )
	{
		if( pposition->_pcollection == pcollection )
		{
			ppositionFound = pposition;
			break;
		}
	}

	if( ppositionFound == NULL)
		throw CollectionException( CE_NOTAMEMBER );

	return pposition->_pcollection->_pNext;
}

Collectable* MultiCollectable::previous( Collection* pcollection )
{
	// Firstly, we iterate through the Positions to make sure that the object is
	//  a member of this collection.
	Position* pposition = NULL;
	Position* ppositionFound = NULL;
	while( ( pposition = (Position*)_collectionS.next( pposition ) ) != NULL )
	{
		if( pposition->_pcollection == pcollection )
		{
			ppositionFound = pposition;
			break;
		}
	}

	if( ppositionFound == NULL)
		throw CollectionException( CE_NOTAMEMBER );

	return pposition->_pcollection->_pPrevious;
}

void MultiCollectable::removeFrom( Collection* pcollection )
{
	// Firstly, we iterate through the Positions to make sure that the object is
	//  a member of this collection.
	Position* pposition = NULL;
	Position* ppositionFound = NULL;
	while( ( pposition = (Position*)_collectionS.next( pposition ) ) != NULL )
	{
		if( pposition->_pcollection == pcollection )
		{
			ppositionFound = pposition;
			break;
		}
	}

	if( ppositionFound == NULL)
		throw CollectionException( CE_NOTAMEMBER );

	// Now we actually remove the object from the collection
	if( pposition->_pcollection->_pPrevious != NULL )
	{
		pposition->_pcollection->_pPrevious->_pNext = pposition->_pcollection->_pNext;
	}
	else
	{
		pposition->_pcollection->_pParent->first( _pNext );
	}

	if( pposition->_pcollection->_pNext != NULL )
	{
		pposition->_pcollection->_pNext->_pPrevious = pposition->_pcollection->_pPrevious;
	}
	else
	{
		pposition->_pcollection->_pParent->last( _pPrevious );
	}
}

void MultiCollectable::remove( long bDelete )
{
	remove( ALL_COLLECTIONS, bDelete );
}

void MultiCollectable::remove( Collection* pcollection, long bDelete )
{
	// Firstly, we iterate through the Positions to make sure that the object is
	//  a member of this collection.
	Position* pposition = NULL;
	Position* ppositionFound = NULL;
	if( pcollection == ALL_COLLECTIONS )
		ppositionFound = (Position*)ALL_COLLECTIONS;

	while( ( pposition = (Position*)_collectionS.next( pposition ) ) != NULL )
	{
		if( pcollection == ALL_COLLECTIONS )
		{
			pposition->_pcollection->remove( this );
		}
		else
		{
			if( pposition->_pcollection == pcollection )
			{
				ppositionFound = pposition;
				break;
			}
		}
	}

	if( pcollection != ALL_COLLECTIONS )
	{
		if( ppositionFound == NULL)
			throw CollectionException( CE_NOTAMEMBER );

		ppositionFound->_pcollection->remove( this );
	}
}

long MultiCollectable::canIDeleteYou() const
{
	// Whether a collection can delete a MultiCollectable is dependent on a number
	//  of factors:

	// If the object is already being deleted, the parent cannot delete it.
	if( _bIsBeingDeleted )
		return 0;

	// The Deletion Policy may state that a collection can never delete this member
	if( !( _lDeletionPolicy & COLDEL_COLLECTIONCANDELETE ) )
		return 0;

	// If the item is no longer a member of any collections (it having already been
	//  removed from this collection), it can be deleted
	if( _collectionS.count() == 0 )
		return 1;

	// If the Deletion Policy states that deletion can take place, thereby causing
	//  the removal of the item from all other collections, it can be deleted
	if( _lDeletionPolicy & COLDEL_REMOVALCASCADE )
		return 1;

	// In all other situations, the Collection cannot delete the item
	return 0;
}
