/*
	$Id: Mutex.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/Mutex.hpp>

USE_RFC_NAMESPACE

Mutex::Mutex()
{
#if defined (LINUX)
	pthread_mutex_init( &_mx, NULL );
#elif defined (_WINDOWS)
	InitializeCriticalSection( &_cs );
#endif /* LINUX */
}

Mutex::~Mutex()
{
#if defined (LINUX)
	pthread_mutex_destroy( &_mx );
#elif defined (_WINDOWS)
	DeleteCriticalSection( &_cs );
#endif /* LINUX */
}

void Mutex::lock()
{
#if defined (LINUX)
	pthread_mutex_lock( &_mx );
#elif defined (_WINDOWS)
	EnterCriticalSection( &_cs );
#endif
}

void Mutex::unlock()
{
#if defined (LINUX)
	pthread_mutex_unlock( &_mx );
#elif defined (_WINDOWS)
	LeaveCriticalSection( &_cs );
#endif
}
