/*
	$Id: ODBCEnvironment.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/ODBCEnvironment.hpp>
#include <librfc/RFCAssert.h>

#include <librfc/Socket.hpp>

USE_RFC_NAMESPACE

ODBCEnvironment::ODBCEnvironment()
{

	_bEnvValid = 0;
	_hEnv = SQL_NULL_HANDLE;

	SQLRETURN rc;
	_bEnvValid = true;

	rc = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &_hEnv );
	if( ( rc == SQL_SUCCESS ) ||
		( rc == SQL_SUCCESS_WITH_INFO ) )
	{
		rc = SQLSetEnvAttr ( _hEnv, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 0 );
	}
	else
	{
		ASSERT( 0 );
		_bEnvValid = false;
	}
}

ODBCEnvironment TAG_RFC_NAMESPACE _ODBCEnvironment;

ODBCEnvironment::~ODBCEnvironment()
{
	if( _bEnvValid )
	{
		SQLFreeHandle( SQL_HANDLE_ENV, _hEnv );
	}
}

SQLHANDLE ODBCEnvironment::handle()
{
	return _hEnv;
}

bool ODBCEnvironment::isEnvValid()
{
	return _bEnvValid;
}

void ODBCEnvironment::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl;
}

void ODBCEnvironment::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n";
}
