/*
	$Id: PooledDB.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/PooledDB.hpp>
#include <librfc/DBPoolException.hpp>
#include <librfc/DatabaseException.hpp>

#include <librfc/Socket.hpp>

USE_RFC_NAMESPACE

PooledDB::PooledDB( const char* pszSource, const char* pszUserId, const char* pszPassword )
		:_pdbNext( NULL ),
		_lInUse( 0 ),
		_lRefresh( 0 )
{
	if( _db.open( pszSource, pszUserId, pszPassword ) < 0 )
	{
		DatabaseException e( _db.lastError() );
//		DBPoolException e( DBPE_OPENFAILED );
		throw e;
	}
}

PooledDB::~PooledDB()
{
	_db.close();

	if( _pdbNext != NULL )
		delete _pdbNext;
}

void PooledDB::setRefresh()
{
	_lRefresh = 1;
}

void PooledDB::refresh( const char* pszSource, const char* pszUserId, const char* pszPassword )
{
	if( _db.isOpen() )
		_db.close();

	if( _db.open( pszSource, pszUserId, pszPassword ) < 0 )
	{
		DatabaseException e( _db.lastError() );
		throw e;
	}

	_lRefresh = 0;
}

void PooledDB::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl;
}

void PooledDB::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n";
}
