/*
	$Id: Profile.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Profile_hpp__
#define __Profile_hpp__

#include <librfc/RFCBase.hpp>
#include <librfc/export.h>

#include <stdio.h>

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export Profile : public RFCBase
{
public:
	class ProfileEntry
	{
	public:
		char* _pszName;
		char* _pszValue;
		bool _bModified;
		bool _bGlobal;
		ProfileEntry* _pnext;

		ProfileEntry( const char* pszName, const char* pszValue );
		ProfileEntry( const char* pszName, long lValue );
		virtual ~ProfileEntry();

		void write( FILE* h );

		const char* name() const;
		const char* valueString() const;
		long valueLong() const;

		void setValue( const char* pszValue );
		void setValue( long lValue );
	};

	class ProfileSection
	{
	public:
		ProfileEntry* _pfirst;
		ProfileEntry* _plast;
		ProfileSection* _pnext;
		char* _pszName;

		ProfileSection( const char* pszName );
		virtual ~ProfileSection();

		void write( FILE* h );

		const char* entryString( const char* pszEntryName, const char* pszDefault ) const;
		long entryLong( const char* pszEntryName, long lDefault ) const;

		void setEntryValue( const char* pszEntryName, const char* pszValue );
		void setEntryValue( const char* pszEntryName, long lValue );

		ProfileEntry* find( const char* pszEntryName ) const;
		void addEntry( const char* pszEntryName, const char* pszValue );
		void addEntry( const char* pszEntryName, long lValue );
	};

	typedef enum { reading_global, reading_local, running, writing } Mode;

private:
	static Mode _mode;
	ProfileSection* _pfirst;
	ProfileSection* _plast;
	char* _pszProfileLocal;
	char* _pszProfileGlobal;
	bool _bProfileFound;

public:
/*
	Linux
	ProfileLocal and ProfileGlobal are the names of the configuration files
	ProfileLocal will be searched for in the current user's home directory.
	ProfileGlobal will be searched for in /etc
*/
	Profile( const char* pszProfileLocal, const char* pszProfileGlobal );
	virtual ~Profile();

	const char* getProfileString( const char* pszSection, const char* pszEntry, const char* pszDefault );
	long getProfileLong( const char* pszSection, const char* pszEntry, long lDefault );
	void writeProfileString( const char* pszSection, const char* pszEntry, const char* pszValue );
	void writeProfileLong( const char* pszSection, const char* pszEntry, long lValue );

	static Mode mode();
	bool profileFound() const;

protected:
	ProfileSection* addSection( const char* pszName );
	ProfileSection* find( const char* pszName ) const;
	void processFile( FILE* h );

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__Profile_hpp__ */
