/*
	$Id: RFCBase.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/RFCBase.hpp>
#include <librfc/Socket.hpp>

#include <iostream>

USE_RFC_NAMESPACE

std::map<RFCBase*, RFCDump> RFCBase::_dumpS;
bool RFCBase::_bInitialised = false;

void RFCDump::dump( std::ostream& os )
{
	_pbase->dump( os );
}

void RFCDump::dump( Socket& skt )
{
	_pbase->dump( skt );
}




#ifdef TRACK_LEAKS
RFCBase::RFCBase()
	: _bExcludeFromDump( false )
{
	if( _bInitialised )
	{
		RFCDump dump( this );
		_dumpS[this] = dump;
	}
}

RFCBase::~RFCBase()
{
	_dumpS.erase( this );
}
#endif /* TRACK_LEAKS */

void RFCBase::excludeFromDump()
{
	_bExcludeFromDump = true;
}

void RFCBase::dumpAll( std::ostream& os )
{
#ifdef TRACK_LEAKS
	if( !_bInitialised )
		os << "RFCBase not initialised!\n";
	else
	{
		os << "Dumping " << _dumpS.size() << " objects.\n";

		for( std::map<RFCBase*, RFCDump>::iterator iter = _dumpS.begin();
			iter != _dumpS.end(); ++iter )
			if( !iter->first->_bExcludeFromDump )
				iter->second.dump( os );
	}
#else
	os << "Leak tracking disabled. Try Adding -DTRACK_LEAKS to the compiler command line\n";
#endif /* TRACK_LEAKS */
}

void RFCBase::dumpAll( Socket& skt )
{
#ifdef TRACK_LEAKS
	if( !_bInitialised )
		skt << "RFCBase not initialised!\n" << Socket::xmit;
	else
	{
		skt << "Dumping " << (unsigned long)( _dumpS.size() ) << " objects.\n";

		for( std::map<RFCBase*, RFCDump>::iterator iter = _dumpS.begin();
			iter != _dumpS.end(); ++iter )
			if( !iter->first->_bExcludeFromDump )
				iter->second.dump( skt );

		skt << Socket::xmit;
	}
#else
	skt << "Leak tracking disabled. Try Adding -DTRACK_LEAKS to the compiler command line\n" << Socket::xmit;
#endif /* TRACK_LEAKS */
}
