/*
	$Id: RFCBase.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __RFCBase_hpp__
#define __RFCBase_hpp__

#include <librfc/export.h>

#include <map>
#include <ostream>
#include <typeinfo>

BEGIN_RFC_NAMESPACE
class RFCBase;
class Socket;

class RFC_Export RFCDump
{
	private:
		RFCBase* _pbase;

	public:
		RFCDump() : _pbase( NULL ){}
		RFCDump( RFCBase* pbase ) : _pbase( pbase ){};
		RFCDump( const RFCDump& dump ) : _pbase( dump._pbase ){}
		virtual ~RFCDump(){}
		const RFCDump& operator = ( const RFCDump& dump ){ _pbase = dump._pbase; return *this; }
		void dump( std::ostream& os );
		void dump( Socket& skt );
};

class RFC_Export RFCBase
{
	private:
		static std::map<RFCBase*, RFCDump> _dumpS;
		static bool _bInitialised;
		bool _bExcludeFromDump;

	protected:
#ifdef TRACK_LEAKS
		RFCBase();
		virtual ~RFCBase();
#else
		RFCBase() : _bExcludeFromDump( false ){}
		virtual ~RFCBase(){}
#endif /* TRACK_LEAKS */

		virtual void dump( std::ostream& os ) = 0;
		virtual void dump( Socket& skt ) = 0;

	public:
		static void initialize(){ _bInitialised = true; }
		static void dumpAll( std::ostream& os );
		static void dumpAll( Socket& skt );

		void excludeFromDump();

		friend class RFCDump;
};
END_RFC_NAMESPACE

#endif /* !__RFCBase_hpp__ */
