/*
	$Id: ResultSet.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/ResultSet.hpp>

#include <librfc/DatabaseException.hpp>

#include <librfc/rfcversion.h>

#include <librfc/Socket.hpp>

#if defined (_WINDOWS)
#	include <windows.h>
//#	include <sql.h>
#	include <sqlext.h>
#	if defined( _DEBUG ) && !defined( _NO_ASSERT )
#		include <assert.h>
#		define ASSERT(x) assert(x)
#	else
#		define ASSERT(x)
#	endif
#elif defined (LINUX)
#	include <stdlib.h>
#	include <stdio.h>
#	include <sql.h>
#	include <sqlext.h>
#	include <sqltypes.h>
#	include <librfc/export.h>
#	include <unistd.h>
#	if defined( _DEBUG ) && !defined( _NO_ASSERT )
#		include <assert.h>
#		define ASSERT(x) assert(x)
#	else
#		define ASSERT(x)
#	endif
#endif

#include <stdio.h>


USE_RFC_NAMESPACE

ResultSet::ResultSet( Database* pdb )
			:_hStmt( SQL_NULL_HSTMT )
{
	_lStmtLength = 0;
	_pdb = pdb;
	_bIsOpen = FALSE;
	_bIsEOF = FALSE;
}

ResultSet::~ResultSet()
{
	if( _bIsOpen )
		close();
}

long ResultSet::setFilter( const String& strFilter )
{
	_strFilter = strFilter;

	return 1;
}

long ResultSet::setSort( const String& strSort )
{
	_strSort = strSort;

	return 1;
}

long ResultSet::open( const String& strStatement )
{
	long lRtn = 0;
	SQLRETURN rc;

	_bIsEOF = FALSE;

	rc = SQLAllocHandle( SQL_HANDLE_STMT, _pdb->_hConn, &_hStmt );
//	rc = SQLAllocStmt( _pdb->_hConn, &_hStmt );
	if( ( rc != SQL_SUCCESS ) &&
		( rc != SQL_SUCCESS_WITH_INFO ) )
	{
		LONG lOutInt;
		SHORT nTextOut;
		SQLTCHAR szOutState[6];
		SQLTCHAR szOutMessage[400];

		ASSERT( 0 );

		SQLGetDiagRec( SQL_HANDLE_DBC, _pdb->_hConn, 1, szOutState,
						(SQLINTEGER*)&lOutInt, szOutMessage, 400, &nTextOut );

		_pdb->_strLastError = (const char*)szOutMessage;

		DatabaseException e( _pdb->_strLastError );
		throw e;

		lRtn = -2;
		return lRtn;		// Exit Point
	}

	if( strStatement.length() )
	{
		_strStmt = strStatement;
	}
	else
	{
		bind( bindSelect );
		String strTemp( _strStmt );
		_strStmt = "SELECT ";
		_strStmt += strTemp;
		_strStmt += " FROM ";
		_strStmt += table();
//		if( ( (const char*)_strFilter != NULL ) && _strFilter.length() )
		if( _strFilter.length() )
		{
			_strStmt += " WHERE ";
			_strStmt += _strFilter;
		}
//		if( ( (const char*)_strSort != NULL ) && _strSort.length() )
		if( _strSort.length() )
		{
			_strStmt += " ORDER BY ";
			_strStmt += _strSort;
		}
	}

	rc = SQLExecDirect( _hStmt, (LPBYTE)(const char*)_strStmt, _strStmt.length() );
	if( ( rc == SQL_SUCCESS ) ||
		( rc == SQL_SUCCESS_WITH_INFO ) )
	{
		if( rc == SQL_SUCCESS_WITH_INFO )
		{
			getCommunication();
		}
		_uColIdx = 1;
		_bIsOpen = TRUE;
		bind( bindData );
	}
	else
	{
		getCommunication();
		lRtn = -1;

		SQLFreeStmt( _hStmt, SQL_UNBIND | SQL_CLOSE | SQL_DROP );
		_hStmt = SQL_NULL_HSTMT;

		DatabaseException e( _pdb->_strLastError );
		throw e;
	}

	return lRtn;
}

long ResultSet::deleteRows()
{
	long lRtn = 0;

	_strStmt = "DELETE FROM ";
	_strStmt += table();
	if( _strFilter.length() )
	{
		_strStmt += " WHERE ";
		_strStmt += _strFilter;
	}

	_pdb->_bInTransaction = 1;

	return lRtn;
}

long ResultSet::updateRows()
{
	long lRtn = 0;

	bind( bindUpdate );
	String strTemp( _strStmt );
	_strStmt = "UPDATE ";
	_strStmt += table();
	_strStmt += " SET ";
	_strStmt += strTemp;
	if( _strFilter.length() )
	{
		_strStmt += " WHERE ";
		_strStmt += _strFilter;
	}

	_pdb->_bInTransaction = 1;

	return lRtn;
}

long ResultSet::insertRow()
{
	long lRtn = 0;

	bind( bindSelect );
	String strTempC( _strStmt );
	_strStmt = "INSERT INTO ";
	_strStmt += table();
	_strStmt += " ( ";
	_strStmt += strTempC;
	_strStmt += " ) VALUES ( ";

	strTempC = _strStmt;

	_strStmt = "";
	bind( bindInsert );

	String strTempD( _strStmt );
	_strStmt = strTempC;
	_strStmt += strTempD;
	_strStmt += " )";

	if( _strFilter.length() )
	{
		_strStmt += " WHERE ";
		_strStmt += _strFilter;
	}

	_pdb->_bInTransaction = 1;

	return lRtn;
}

long ResultSet::bindColumn( bindType typeBind, const char* pszColumn, void* pvData, bindDataType typeData, int nBufferLength )
{
	long lRtn = 0;
	String strBit;
	int nTargetType;

	switch( typeBind )
	{
		case bindSelect:
			if( _strStmt.length() )
			{
				_strStmt += ", ";
			}
			_strStmt += pszColumn;
			break;

		case bindInsert:
			if( _strStmt.length() )
			{
				_strStmt += ", ";
			}
			switch( typeData )
			{
				case bindSShort:
					strBit.format( "%i", pszColumn, (short)(*(short*)pvData) );
					_strStmt += strBit;
					break;

				case bindUShort:
					strBit.format( "%u", pszColumn, (short)(*(short*)pvData) );
					_strStmt += strBit;
					break;

				case bindSLong:
					strBit.format( "%li", pszColumn, (long)(*(long*)pvData) );
					_strStmt += strBit;
					break;

				case bindULong:
					strBit.format( "%lu", pszColumn, (long)(*(long*)pvData) );
					_strStmt += strBit;
					break;

				case bindChar:
					strBit.format( "'%s'", pszColumn, (const char*)pvData );
					_strStmt += strBit;
					break;
			}
			break;

		case bindUpdate:
			if( _strStmt.length() )
			{
				_strStmt += ", ";
			}
			switch( typeData )
			{
				case bindSShort:
					strBit.format( "%s = %i", pszColumn, (short)(*(short*)pvData) );
					_strStmt += strBit;
					break;

				case bindUShort:
					strBit.format( "%s = %u", pszColumn, (short)(*(short*)pvData) );
					_strStmt += strBit;
					break;

				case bindSLong:
					strBit.format( "%s = %li", pszColumn, (long)(*(long*)pvData) );
					_strStmt += strBit;
					break;

				case bindULong:
					strBit.format( "%s = %lu", pszColumn, (long)(*(long*)pvData) );
					_strStmt += strBit;
					break;

				case bindChar:
					strBit.format( "%s = '%s'", pszColumn, (const char*)pvData );
					_strStmt += strBit;
					break;
			}
			break;

/*
		SQL_C_CHAR			SQLCHAR *			unsigned char * 
		SQL_C_SSHORT [j]	SQLSMALLINT			short int 
		SQL_C_USHORT [j]	SQLUSMALLINT		unsigned short int 
		SQL_C_SLONG [j]		SQLINTEGER			long int 
		SQL_C_ULONG [j]		SQLUINTEGER			unsigned long int 
		SQL_C_FLOAT			SQLREAL				float 
		SQL_C_DOUBLE		SQLDOUBLE, SQLFLOAT	double 
		SQL_C_BIT			SQLCHAR				unsigned char 
		SQL_C_STINYINT [j]	SQLSCHAR			signed char 
		SQL_C_UTINYINT [j]	SQLCHAR				unsigned char 
		SQL_C_SBIGINT		SQLBIGINT			_int64 [h] 
		SQL_C_UBIGINT		SQLUBIGINT			unsigned _int64 [h] 
		SQL_C_BINARY		SQLCHAR *			unsigned char * 
		SQL_C_BOOKMARK [i]	BOOKMARK			unsigned long int [d] 
		SQL_C_VAR-BOOKMARK	SQLCHAR *			unsigned char * 
		SQL_C_TYPE_DATE [c]	SQL_DATE_STRUCT		struct tagDATE_STRUCT {
																		SQLSMALLINT year; 
																		SQLUSMALLINT month; 
																		SQLUSMALLINT day; 
																		} DATE_STRUCT; [a] 
		SQL_C_TYPE_TIME [c]	SQL_TIME_STRUCT		struct tagTIME_STRUCT {
																		SQLUSMALLINT hour; 
																		SQLUSMALLINT minute; 
																		SQLUSMALLINT second; 
																		} TIME_STRUCT; [a] 
		SQL_C_TYPE_TIMESTAMP [c]	SQL_TIMESTAMP_STRUCT	struct tagTIMESTAMP_STRUCT {
																						SQLSMALLINT year; 
																						SQLUSMALLINT month; 
																						SQLUSMALLINT day; 
																						SQLUSMALLINT hour; 
																						SQLUSMALLINT minute; 
																						SQLUSMALLINT second; 
																						SQLUINTEGER fraction; [b]
																						} TIMESTAMP_STRUCT; [a] 
		SQL_C_NUMERIC		SQL_NUMERIC_STRUCT	struct tagSQL_NUMERIC_STRUCT {
																				SQLCHAR precision;
																				SQLSCHAR scale; 
																				SQLCHAR sign [g]; 
																				SQLCHAR
																				val[SQL_MAX_NUMERIC_LEN]; [e], [f] 
																				} SQL_NUMERIC_STRUCT; 
		SQL_C_GUID			SQLGUID				struct tagSQLGUID {
																	DWORD Data1;
																	WORD Data2;
																	WORD Data3;
																	BYTE Data4[8];
																	} SQLGUID; [k] 
*/
		case bindData:
			switch( typeData )
			{
				case bindChar:
					nTargetType = SQL_C_CHAR;
					break;

				case bindSShort:
					nTargetType = SQL_C_SSHORT;
					break;

				case bindUShort:
					nTargetType = SQL_C_USHORT;
					break;

				case bindSLong:
					nTargetType = SQL_C_SLONG;
					break;

				case bindULong:
					nTargetType = SQL_C_ULONG;
					break;

				case bindFloat:
					nTargetType = SQL_C_FLOAT;
					break;

				case bindDouble:
					nTargetType = SQL_C_DOUBLE;
					break;

				case bindBit:
					nTargetType = SQL_C_BIT;
					break;

				case bindSTinyInt:
					nTargetType = SQL_C_STINYINT;
					break;

				case bindUTinyInt:
					nTargetType = SQL_C_UTINYINT;
					break;

				case bindSBigInt:
					nTargetType = SQL_C_SBIGINT;
					break;

				case bindUBigInt:
					nTargetType = SQL_C_UBIGINT;
					break;

				case bindBinary:
					nTargetType = SQL_C_BINARY;
					break;

				case bindDate:
					nTargetType = SQL_C_TYPE_DATE;
					break;

				case bindTime:
					nTargetType = SQL_C_TYPE_TIME;
					break;

				case bindTimestamp:
					nTargetType = SQL_C_TYPE_TIMESTAMP;
					break;

				case bindNumeric:
					nTargetType = SQL_C_NUMERIC;
			}

			SQLBindCol( _hStmt, _uColIdx++, nTargetType, pvData, nBufferLength, &_lSLen );
			break;
	}

	return lRtn;
}

long ResultSet::close()
{
	long lRtn = 0;
	SQLRETURN rc;

	if( !_bIsOpen )
		return lRtn;

	_bIsOpen = FALSE;

/*	rc = SQLCloseCursor( _hStmt );
	if( ( rc != SQL_SUCCESS ) &&
		( rc != SQL_SUCCESS_WITH_INFO ) )
	{
		getCommunication();
	}*/

	rc = SQLFreeStmt( _hStmt, SQL_UNBIND );
	rc = SQLFreeStmt( _hStmt, SQL_RESET_PARAMS );
	rc = SQLFreeStmt( _hStmt, SQL_CLOSE );

	rc = SQLFreeHandle( SQL_HANDLE_STMT, &_hStmt );
	if( ( rc != SQL_SUCCESS ) &&
		( rc != SQL_SUCCESS_WITH_INFO ) )
	{
		getCommunication();
	}

	_hStmt = SQL_NULL_HSTMT;

	return lRtn;
}

long ResultSet::fetch()
{
	long lRtn = -1;
	SQLRETURN rc;

	clear();

	rc = SQLFetch( _hStmt );
	switch( rc )
	{
		case SQL_NO_DATA:
			lRtn= 1;
			_bIsEOF = TRUE;
			break;

		case SQL_SUCCESS_WITH_INFO:
			getCommunication();
		case SQL_SUCCESS:
			lRtn = 0;
			break;

		default:
		{
			getCommunication();
			lRtn = -1;
		}
	}

	return lRtn;
}

//long ResultSet::expandStmt( const char* pszText )
//{
//	return _lStmtLength;
//}

long ResultSet::getCommunication()
{
	LONG lOutInt;
	SHORT nTextOut;
	SQLTCHAR szOutState[6];
	SQLTCHAR szOutMessage[400];

	SQLGetDiagRec( SQL_HANDLE_STMT, _hStmt, 1, szOutState,
					(SQLINTEGER*)&lOutInt, szOutMessage, 400, &nTextOut );

	_pdb->_strLastError = (const char*)szOutMessage;

	return lOutInt;
}

long ResultSet::rowCount()
{
	SQLLEN nCount;

	SQLRowCount( _hStmt, &nCount );

	return nCount;
}

void ResultSet::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl
		<< ( _strStmt.isNull() ? "(null)" : (const char*)_strStmt )
		<< std::endl;
}

void ResultSet::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n"
		<< ( _strStmt.isNull() ? "(null)" : (const char*)_strStmt )
		<< "\n";
}
