/*
	$Id: ResultSet.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __RESULTSET_HPP__
#define __RESULTSET_HPP__

#include <librfc/export.h>
#include <librfc/String.hpp>
#include <librfc/Database.hpp>
#include <librfc/RFCBase.hpp>

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export ResultSet : public RFCBase
{
private:
	unsigned short	_uColIdx;
	SQLLEN			_lSLen;
	long			_bIsOpen;
	long			_bIsEOF;
	long			_lStmtLength;
	String			_strStmt;
	String			_strFilter;
	String			_strSort;
	SQLHANDLE		_hStmt;
	Database*		_pdb;

	ResultSet();

public:
	typedef enum{ bindChar, bindSShort, bindUShort, bindSLong, bindULong,
					bindFloat, bindDouble, bindBit, bindSTinyInt, bindUTinyInt,
					bindSBigInt, bindUBigInt, bindBinary, bindDate, bindTime,
					bindTimestamp, bindNumeric }bindDataType;

protected:
	typedef enum{ bindSelect, bindInsert, bindUpdate, bindData }bindType;
	long bindColumn( bindType typeBind, const char* pszColumn, void* pvData, bindDataType typeData, int nBufferLength );
	long getCommunication();

public:
	ResultSet( Database* pdb );
	virtual ~ResultSet();

	virtual void clear() = 0;
	long setFilter( const String& strFilter = String() );
	long setSort( const String& strSort = String() );
	long open( const String& strStatement = String() );
	long deleteRows();
	long updateRows();
	long insertRow();
	virtual long bind( bindType type ) = 0;
	long close();
	long fetch();
	long isEOF(){ return _bIsEOF; }
	long isOpen(){ return _bIsOpen; }
	virtual const String table() = 0;
	long rowCount();

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__RESULTSET_HPP__ */
