/*
	$Id: Socket.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Socket_hpp__
#define __Socket_hpp__

#include <librfc/String.hpp>
#include <librfc/export.h>
#include <librfc/RFCBase.hpp>
//#include <iostream>


BEGIN_RFC_NAMESPACE
class RFC_Export Socket : public RFCBase
{
public:
	typedef enum { Inet, Unix } Domain;
	typedef enum { xmit } Action;

private:
	int _s;
	Domain _domain;
	String _strWaiting;

private:
	Socket( int s );

public:
	Socket( Domain domain = Inet );
	Socket( const Socket& skt );
	~Socket();

	const Socket& operator= ( const Socket& skt );

	bool bind( short nPort );
	bool bind( const String& strFile );
	bool listen();
	Socket accept();
	bool connect( const String& strHost, short nPort );
	bool connect( const String& strFile );

	unsigned long send( const String& strData );
	unsigned long recv( String& strData, unsigned long ulTimeout = (unsigned long)-1 );
	void close();

//	int s() const { return _s; }

	static const char* resolveSocketError( unsigned long ulError );

	Socket& operator << ( const char* psz );
	Socket& operator << ( short n );
	Socket& operator << ( unsigned short u );
	Socket& operator << ( long l );
	Socket& operator << ( unsigned long ul );
	Socket& operator << ( Action a );

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__Socket_hpp__ */
