/*
	$Id: String.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/String.hpp>
#include <librfc/StringException.hpp>
#include <librfc/Database.hpp>
#include <librfc/Variable.hpp>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <librfc/rfcversion.h>
#if defined (LINUX)
#	include <unistd.h>
#endif

#define MASK_POSITIVE	'#'
#define MASK_NEGATIVE	' '

#include <librfc/Socket.hpp>

/*typedef struct __tagEscapeMap
{
	char cChar;
	char* pszCode;
	size_t lLen;
} EscapeMap;*/

/*
	Note: When adding codes to the map, be sure to add them
	to the CodedS array as well. Codes added to the map must
	appear before the terminating NULL entry.
*/
/*static EscapeMap map[] = {	{ '\n', "\\n", 2 },
							{ '\t', "\\t", 2 },
							{ '\v', "\\v", 2 },
							{ '\b', "\\b", 2 },
							{ '\r', "\\r", 2 },
							{ '\f', "\\f", 2 },
							{ '\a', "\\a", 2 },
							{ '\\', "\\\\", 2 },
							{ '\'', "\\\'", 2 },
							{ '\"', "\\\"", 2 },
							{ 0, NULL, 0 } };

static char* pszEscs = "\n\t\v\b\r\f\a\\'\"";*/

USE_RFC_NAMESPACE

const String String::null = String();

String::String()
	: _bIsNumericData( false ),
	_pszSQL( NULL )
{
//	printf( "String::String()\n" );

	_ulFixedLength = 0;
	_ulDataLength = 0;
	_bIsNull = true;
	_pszData = 0;
}

String::String( const String& strString )
	: _bIsNumericData( strString._bIsNumericData ),
	_pszSQL( NULL )
{
//	printf( "String::String( const String& strString = %s )\n", (const char*)strString );

	_ulFixedLength = strString._ulFixedLength;
	_ulDataLength = strString._ulDataLength;
	_bIsNull = strString._bIsNull;

	if( _ulDataLength )
	{
		_pszData = new char[_ulDataLength];
		if( _pszData == NULL )
		{
			StringException e( SE_MEMORY );
			throw e;
		}
		strcpy( _pszData, strString._pszData );
	}
	else
		_pszData = NULL;
}

String::String( const char* pszString, unsigned long ulFixedLength /*= 0*/ )
	: _ulFixedLength( ulFixedLength ),
	_bIsNumericData( false ),
	_pszSQL( NULL )
{
//	printf( "String::String( const char* pszString = %s )\n", pszString );

	if( _ulFixedLength )
	{
		_ulDataLength = _ulFixedLength + 1;
		_pszData = new char[_ulDataLength];

		if( _pszData == NULL )
		{
			StringException e( SE_MEMORY );
			throw e;
		}

		*_pszData = '\0';
	}
	else
	{
		_ulDataLength = 0;
		_pszData = NULL;
	}

	if( pszString == NULL )
	{
		_bIsNull = true;
	}
	else
	{
		_bIsNull = false;
		unsigned long ulLength = strlen( pszString );
		if( _ulFixedLength )
		{
			if( ulLength > _ulDataLength )
			{
				StringException e( SE_LENGTH );
				throw e;
			}
		}
		else
			_ulDataLength = ulLength + 1;

		if( _pszData == NULL )
			_pszData = new char[_ulDataLength];

		if( _pszData == NULL )
		{
			StringException e( SE_MEMORY );
			throw e;
		}

		strcpy( _pszData, pszString );
	}
}

String::String( unsigned long ulFixedLength )
	: _ulFixedLength( ulFixedLength ),
	_bIsNumericData( false ),
	_pszSQL( NULL )
{
	_ulDataLength = _ulFixedLength + 1;

	_pszData = new char[_ulDataLength];
	if( _pszData == NULL )
	{
		StringException e( SE_MEMORY );
		throw e;
	}

	*_pszData = '\0';
	_bIsNull = true;
}

String::~String()
{
	if( _pszData != NULL )
		delete[] _pszData;
	if( _pszSQL != NULL )
		delete[] _pszSQL;
}

bool String::isNull() const
{
	return _bIsNull;
}

const String& String::copy( const String& strString, unsigned long ulLength /*= -1*/ )
{
	_bIsNumericData = false;

	if( ulLength == -1UL )
		ulLength = strString.length();

	char* pszTemp = new char[( ulLength + 1 )];
	if( pszTemp == NULL )
	{
		StringException e( SE_MEMORY );
		throw e;
	}
	strncpy( pszTemp, strString, ulLength );
	pszTemp[ulLength] = 0;

	try
	{
		(*this) = pszTemp;
	}
	catch( ... )
	{
		delete[] pszTemp;
		throw;
	}

	delete[] pszTemp;

	return *this;
}

const String& String::copy( const char* pszString, unsigned long ulLength /*= -1*/ )
{
	String strString( pszString );
	return copy( strString, ulLength );
}

const String& String::operator = ( const String& strString )
{
	if( &strString == this )
		return *this;

	_bIsNumericData = strString._bIsNumericData;

	if( _pszData != NULL )
	{
		delete[] _pszData;
		_pszData = NULL;
	}

	_ulFixedLength = strString._ulFixedLength;
	_ulDataLength = strString._ulDataLength;
	_bIsNull = strString._bIsNull;

	if( _ulDataLength )
	{
		_pszData = new char[_ulDataLength];
		if( _pszData == NULL )
		{
			StringException e( SE_MEMORY );
			throw e;
		}
		strcpy( _pszData, strString._pszData );
	}
	else
		_pszData = NULL;

	if( ( _pszData != NULL ) &&
		( strString._pszData != NULL ) )
		strcpy( _pszData, strString._pszData );

	return *this;
}

const String& String::operator = ( const char* pszString )
{
	_bIsNumericData = false;

	String strString( pszString );
	return (*this) = strString;
}

const String& String::operator += ( const String& strString )
{
	_bIsNumericData = false;

	unsigned long ulLength = length() + strString.length();

	if( strString._bIsNull )
		return *this;

	if( _ulFixedLength )
	{
		if( ulLength > ( _ulFixedLength ) )
		{
			StringException e( SE_LENGTH );
			throw e;
		}
		else
		{
			if( strString._pszData != NULL )
				strcat( _pszData, strString._pszData );
		}
	}
	else
	{
		_ulDataLength = ulLength + 1;
		char* pszTemp = new char[_ulDataLength];
		if( pszTemp == NULL )
		{
			StringException e( SE_MEMORY );
			throw e;
		}

		*pszTemp = '\0';

		if( _pszData != NULL )
			strcpy( pszTemp, _pszData );
		if( strString._pszData != NULL )
		{
			char* pszEnd = pszTemp + strlen( pszTemp );
			strcpy( pszEnd, strString._pszData );
//			strcat( pszTemp, strString._pszData );
		}

		if( _pszData != NULL )
			delete[] _pszData;

		_bIsNull = false;

		_pszData = pszTemp;
	}

	return *this;
}

const String& String::operator += ( const char* pszString )
{
	_bIsNumericData = false;

	if( pszString == NULL )
		return *this;

	String strTemp( pszString );
	return (*this) += strTemp;
}

const String& String::operator += ( char cCh )
{
	_bIsNumericData = false;

	char sz[2];
	sz[0] = cCh;
	sz[1] = '\0';
	return (*this) += sz;
}

const String String::operator + ( const String& strString )
{
	String strTemp( _pszData );

	strTemp += strString;

	return strTemp;
}

const String String::operator + ( const char* pszString )
{
	String strTemp( _pszData );

	strTemp += pszString;

	return strTemp;
}

const String String::operator + ( char cCh )
{
	String strTemp( _pszData );

	strTemp += cCh;

	return strTemp;
}

bool String::operator == ( const String& strString ) const
{
	if( _bIsNull && strString._bIsNull )
		return 1;

	if( _bIsNull || strString._bIsNull )
		return 0;

	return !strcmp( _pszData, strString._pszData );
}

bool String::operator == ( const char* pszString ) const
{
	if( _bIsNull && ( pszString == NULL ) )
		return 1;

	if( _bIsNull )
		return 0;

	return !strcmp( _pszData, pszString );
}

bool String::operator != ( const String& strString ) const
{
	if( _bIsNull && strString._bIsNull )
		return 0;

	if( _bIsNull || strString._bIsNull )
		return 1;

	return strcmp( _pszData, strString._pszData );
}

bool String::operator != ( const char* pszString ) const
{
	if( _bIsNull && ( pszString == NULL ) )
		return 0;

	if( _bIsNull )
		return 1;

	return strcmp( _pszData, pszString );
}

int String::compare( const String& strString, bool bCaseInsensitive /*= 0*/ ) const
{
	if( _bIsNull && strString._bIsNull )
		return 0;

	if( _bIsNull )
		return -1;

	if( strString._bIsNull )
		return 1;

	if( bCaseInsensitive )
#if defined (_WINDOWS)
		return stricmp( _pszData, strString._pszData );
#elif defined (LINUX)
		return strcasecmp( _pszData, strString._pszData );
#endif
	else
		return strcmp( _pszData, strString._pszData );
}

int String::compare( const char* pszString, bool bCaseInsensitive /*= 0*/ ) const
{
	String strTemp( pszString );
	return compare( strTemp, bCaseInsensitive );
/*
	if( bCaseInsensitive )
#if defined (_WINDOWS)
		return stricmp( _pszData, pszString );
#elif defined (LINUX)
		return strcmp( _pszData, pszString );
#endif
	else
		return strcmp( _pszData, pszString );
*/
}

int String::compareFor( const String& strString, unsigned long ulLength, bool bCaseInsensitive /*= 0*/ ) const
{
	if( _bIsNull && strString._bIsNull )
		return 0;

	if( _bIsNull )
		return -1;

	if( strString._bIsNull )
		return 1;

	if( bCaseInsensitive )
#if defined (_WINDOWS)
		return strnicmp( _pszData, strString._pszData, ulLength );
#elif defined (LINUX)
		return strncasecmp( _pszData, strString._pszData, ulLength );
#endif
	else
		return strncmp( _pszData, strString._pszData, ulLength );
}

int String::compareFor( const char* pszString, unsigned long ulLength, bool bCaseInsensitive /*= 0*/ ) const
{
	String strTemp( pszString );
	return compareFor( strTemp, ulLength, bCaseInsensitive );
/*
	if( bCaseInsensitive )
#if defined (_WINDOWS)
		return strnicmp( _pszData, pszString, ulLength );
#elif defined (LINUX)
		return strncmp( _pszData, pszString, ulLength );
#endif
	else
		return strncmp( _pszData, pszString, ulLength );
*/
}

void String::fixLength( unsigned long ulFixedLength /*= 0*/ )
{
	if( ( ulFixedLength != 0 ) &&
		( strlen( _pszData ) > ulFixedLength ) )
	{
		StringException e( SE_LENGTH );
		throw e;
	}

	_ulFixedLength = ulFixedLength;

	if( _ulFixedLength != 0 )
		_ulDataLength = ulFixedLength + 1;
	else
		_ulDataLength = strlen( _pszData ) + 1;

	char* pszTemp = new char[_ulDataLength];
	if( pszTemp == NULL )
	{
		StringException e( SE_MEMORY );
		throw e;
	}

	if( _pszData != NULL )
	{
		strcpy( pszTemp, _pszData );
		delete[] _pszData;
		_pszData = NULL;
	}
	_pszData = pszTemp;

	if( _ulFixedLength == 0 )
		release();
}

unsigned long String::fixedLength() const
{
	return _ulFixedLength;
}

char* String::buffer( unsigned long ulLength /*= 0*/ )
{
	if( ulLength != 0 )
	{
		if( _ulFixedLength && ( ulLength > _ulFixedLength ) )
		{
			StringException e( SE_LENGTH );
			throw e;
		}
		else
		{
			_ulDataLength = ulLength + 1;
			char* pszTemp = new char[_ulDataLength];
			if( pszTemp == NULL )
			{
				StringException e( SE_MEMORY );
				throw e;
			}
			if( _pszData != NULL )
			{
				strcpy( pszTemp, _pszData );
				delete[] _pszData;
			}
			_pszData = pszTemp;
		}
	}

	return _pszData;
}

void String::release()
{
	if( _ulFixedLength )
		return;

	_ulDataLength = strlen( _pszData ) + 1;
	char* pszTemp = new char[_ulDataLength];
	if( pszTemp == NULL )
	{
		StringException e( SE_MEMORY );
		throw e;
	}

	if( _pszData != NULL )
	{
		strcpy( pszTemp, _pszData );
		delete[] _pszData;
	}

	_pszData = pszTemp;
}

void String::format( const char* pszFmt, ... )
{
	va_list va;

	va_start( va, pszFmt );

	if( _ulFixedLength == 0 )
	{
		if( _ulDataLength < 256 )
		{
			_ulDataLength = 256;
			if( _pszData != NULL )
				delete[] _pszData;
			_pszData = new char[_ulDataLength + 1];
		}
	}
	if( _pszData == NULL )
	{
		StringException e( SE_MEMORY );
		throw e;
	}
#if defined (_WINDOWS)
	_vsnprintf( _pszData, _ulFixedLength ? _ulFixedLength : _ulDataLength - 1, pszFmt, va );
#elif defined (LINUX)
	vsnprintf( _pszData, _ulFixedLength ? _ulFixedLength : _ulDataLength - 1, pszFmt, va );
#endif

	_bIsNull = false;
}

unsigned long String::length() const
{
	if( _bIsNull || (_pszData == NULL ) )
		return 0;

	return strlen( _pszData );
}

String::operator const char*() const
{
	if( _bIsNull || (_pszData == NULL ) )
		return NULL;

	return (const char*)_pszData;
}

/*char String::operator[]( unsigned long ulPosition ) const
{
	if( ulPosition >= strlen( _pszData ) )
	{
		RStringSubscriptException e;
		throw e;
	}

	return _pszData[ulPosition];
}*/

char String::at( unsigned long ulPosition ) const
{
	if( _pszData == NULL )
	{
		StringException e( SE_SUBSCRIPT );
		throw e;
	}

	if( ulPosition >= strlen( _pszData ) )
	{
		StringException e( SE_SUBSCRIPT );
		throw e;
	}

	return _pszData[ulPosition];
}

const char* String::find( char cCh, bool bReverse /*= 0*/ ) const
{
	if( bReverse )
		return strrchr( _pszData, cCh );
	else
		return strchr( _pszData, cCh );
}

const char* String::find( const char* pszString ) const
{
	return strstr( _pszData, pszString );
}

unsigned long String::findIndex( char cCh, bool bReverse /*= 0*/ ) const
{
	char* pszFound;
	
	if( bReverse )
		pszFound = strrchr( _pszData, cCh );
	else
		pszFound = strchr( _pszData, cCh );

	return pszFound == NULL ? STRING_NOTFOUND : pszFound - _pszData;
}

unsigned long String::findIndex( const char* pszString ) const
{
	char* pszFound = strstr( _pszData, pszString );

	return pszFound == NULL ? STRING_NOTFOUND : pszFound - _pszData;
}

String String::createMask( const String& strSearch ) const
{
	String strRtn( *this );

	char* pszPos = strRtn.buffer();
	unsigned long ulLength = length();

	for( unsigned long ulIndex = 0; ulIndex < ulLength; ulIndex++ )
	{
		if( strchr( strSearch, at( ulIndex ) ) )
		{
			pszPos[ulIndex] = MASK_POSITIVE;
		}
		else
		{
			pszPos[ulIndex] = MASK_NEGATIVE;
		}
	}

	return strRtn;
}

String String::mask( const String& strMask ) const
{
	String strRtn( *this );

	char* pszPos = strRtn.buffer();
	unsigned long ulLength = length();
	if( strMask.length() != ulLength )
	{
		StringException e( SE_MASKLENGTH );
		throw e;
	}

	for( unsigned long ulIndex = 0; ulIndex < ulLength; ulIndex++ )
	{
		if( strMask.at( ulIndex ) == MASK_NEGATIVE )
		{
			pszPos[ulIndex] = MASK_NEGATIVE;
		}
	}

	return strRtn;
}

String String::replaceMask( const String& strMask, const String& strReplacementText ) const
{
	unsigned long ulMaskLength = strMask.trim().length();
	unsigned long ulMaskStart = strMask.find( MASK_POSITIVE ) - (const char*)strMask;

	String strRtn;

	strRtn = mid( 0, ulMaskStart );
	strRtn += strReplacementText;
	strRtn += mid( ulMaskStart + ulMaskLength );

	return strRtn;
}

String String::trimRight() const
{
	String strTemp( *this );
	const char* pszStart = (const char*)strTemp;
	char* pszEnd = (char*)pszStart + strTemp.length() - 1;
	while( ( ( *pszEnd == ' ' ) || ( *pszEnd == '\t' ) ) && ( pszEnd > pszStart ) )
		*pszEnd-- = 0;

	return String( pszStart );
}

String String::trimLeft() const
{
	String strTemp( *this );
	char* pszStart = strTemp.buffer();
	while( ( *pszStart == ' ' ) || ( *pszStart == '\t' ) )
		pszStart++;

	return String( pszStart );
}

String String::trim() const
{
	String strTemp( *this );

	return String( strTemp.trimLeft().trimRight() );
}

void String::unescape()
{
	char* pszPos = _pszData;

	while( ( pszPos = strchr( pszPos, '\\' ) ) != NULL )
	{
		switch( *( pszPos + 1 ) )
		{
			case 'n':
				*pszPos = '\n';
				break;
			case 't':
				*pszPos = '\t';
				break;
			case 'v':
				*pszPos = '\v';
				break;
			case 'b':
				*pszPos = '\b';
				break;
			case 'r':
				*pszPos = '\r';
				break;
			case 'f':
				*pszPos = '\f';
				break;
			case 'a':
				*pszPos = '\a';
				break;
			case '\\':
				*pszPos = '\\';
				break;
			case '?':
				*pszPos = '?';
				break;
			case '\'':
				*pszPos = '\'';
				break;
			case '\"':
				*pszPos = '\"';
				break;
			default:
				throw StringException( SE_INVALIDESCAPE );
		}
		strcpy( pszPos + 1, pszPos + 2 );
		pszPos++;
	}
}

void String::escape()
{
	const char* pszEscs = "\n\t\v\b\r\f\a\\\'\"";
	int nC( 0 );
	char* pszPos = _pszData;
	const char* pszRepl;

	pszPos = strpbrk( pszPos, pszEscs );
	while( pszPos != NULL )
	{
		pszPos++;
		nC++;
		pszPos = strpbrk( pszPos, pszEscs );
	}

	char* pszTarget = new char[( strlen( _pszData ) + nC + 1 )];
	char* pszRemnant = new char[( strlen( _pszData ) + nC + 1 )];

	strcpy( pszTarget, _pszData );
	pszPos = strpbrk( pszTarget, pszEscs );
	while( pszPos != NULL )
	{
		strcpy( pszRemnant, pszPos + 1 );
		switch( *pszPos )
		{
			case '\n':
				pszRepl = "\\n";
				break;
			case '\t':
				pszRepl = "\\t";
				break;
			case '\v':
				pszRepl = "\\v";
				break;
			case '\b':
				pszRepl = "\\b";
				break;
			case '\r':
				pszRepl = "\\r";
				break;
			case '\f':
				pszRepl = "\\f";
				break;
			case '\a':
				pszRepl = "\\a";
				break;
			case '\\':
				pszRepl = "\\\\";
				break;
			case '\'':
				pszRepl = "\\\'";
				break;
			case '\"':
				pszRepl = "\\\"";
				break;
		}

		strcpy( pszPos, pszRepl );
		strcat( pszPos, pszRemnant );

		pszPos = strpbrk( pszPos + 2, pszEscs );
	}

	*this = pszTarget;
	delete[] pszTarget;
	delete[] pszRemnant;
}

void String::toURI()
{
}

void String::fromURI()
{
	char* pszTarget = new char[( length() + 1 )];
	char* pszPosStart = _pszData;
	char* pszPosEnd;
	char* pszTerm;
	char szDigit[4];
	char szChar[2];

	szDigit[2] = '\0';
	szChar[1] = '\0';
	*pszTarget = '\0';

	pszPosEnd = strchr( pszPosStart, '%' );
	while( pszPosEnd != NULL )
	{
		*pszPosEnd = '\0';
		pszPosEnd++;
		strncpy( szDigit, pszPosEnd, 2 );
		strcat( pszTarget, pszPosStart );
		szChar[0] = (char)(long)strtol( szDigit, &pszTerm, 16 );
		strcat( pszTarget, szChar );
		pszPosStart = pszPosEnd + 2;
		pszPosEnd = strchr( pszPosStart, '%' );
	}
	strcat( pszTarget, pszPosStart );

	*this = pszTarget;
	delete[] pszTarget;
}

String String::mid( unsigned long ulStart, unsigned long ulLength ) const
{
	unsigned long ulMyLength = length();

	if( ulLength == (unsigned long)-1 )
	{
		ulLength = ulMyLength - ulStart;
	}

	if( ( ulLength <= 0 ) ||
		( ulLength > (unsigned long)(long)-2 ) )
	{
		return String();
	}

	if( ( ( ulStart + ulLength ) > ulMyLength ) ||
		( ( (long)( ulStart + ulLength ) < 1 ) ) )
	{
		return String();
	}

	String strRtn( &_pszData[ulStart] );

	if( ulLength != (unsigned long)-1 )
	{
		*( strRtn._pszData + ulLength ) = 0;
		strRtn.release();

	}

	return strRtn;
}

String String::field( char cDelimiter, unsigned long ulIndex, char cFieldDelimiter ) const
{
	String strRtn;
	unsigned long ulCurrentField = 0;
	unsigned long ulCurrentChar = 0;
	int bQuoted = 0;
	int bContinue = 1;

	while( ( ulCurrentField < ulIndex ) && bContinue )
	{
		try
		{
			if( ( cFieldDelimiter ) && 
				( at( ulCurrentChar ) == cFieldDelimiter ) )
			{
				bQuoted = !bQuoted;
			}
			else if( ( !bQuoted ) &&
				( at( ulCurrentChar ) == cDelimiter ) )
			{
				ulCurrentField++;
			}
			ulCurrentChar++;
		}
		catch( StringException& e )
		{
			if( e.reason() == SE_SUBSCRIPT )
				bContinue = 0;
		}
	}

	while( bContinue )
	{

		try
		{
			if( ( cFieldDelimiter ) && 
				( at( ulCurrentChar ) == cFieldDelimiter ) )
			{
				bQuoted = !bQuoted;
			}
			else if( ( !bQuoted ) &&
				( at( ulCurrentChar ) == cDelimiter ) )
			{
				bContinue = 0;
			}
			else
			{
				strRtn += at( ulCurrentChar );
			}
			ulCurrentChar++;
		}
		catch( StringException& e )
		{
			if( e.reason() == SE_SUBSCRIPT )
				bContinue = 0;
		}
	}

	return strRtn;
}

String::operator long() const
{

	char* pszTerm;
	return strtol( _pszData, &pszTerm, 10 );
}

String::operator unsigned long() const
{
	char* pszTerm;
	return strtoul( _pszData, &pszTerm, 10 );
}

String::operator double() const
{
	char* pszTerm;
	return strtod( _pszData, &pszTerm );
}

const char* String::sql()
{
	if( _pszSQL != NULL )
	{
		delete[] _pszSQL;
		_pszSQL = NULL;
	}

	if( _bIsNull )
	{
		_pszSQL = new char[8];
		strcpy( _pszSQL, "NULL" );
	}
	else if( _bIsNumericData )
	{
		return (const char*)_pszData;
	}
	else
	{
		char* pszTemp = new char[( length() * 2 )];
		Database::DeQuote( pszTemp, _pszData );
		_pszSQL = new char[( strlen( pszTemp ) + 8 )];
		sprintf( _pszSQL, "'%s'", pszTemp );
		delete[] pszTemp;
	}

	return _pszSQL;
}

const String& String::operator = ( signed char cValue )
{
	char szTemp[128];
	sprintf( szTemp, "%i", cValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

const String& String::operator = ( unsigned char byValue )
{
	char szTemp[128];
	sprintf( szTemp, "%u", byValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

const String& String::operator = ( signed short nValue )
{
	char szTemp[128];
	sprintf( szTemp, "%i", nValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

const String& String::operator = ( unsigned short uValue )
{
	char szTemp[128];
	sprintf( szTemp, "%u", uValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

const String& String::operator = ( signed long lValue )
{
	char szTemp[128];
	sprintf( szTemp, "%li", lValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

const String& String::operator = ( unsigned long ulValue )
{
	char szTemp[128];
	sprintf( szTemp, "%lu", ulValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

const String& String::operator = ( float fValue )
{
	char szTemp[128];
	sprintf( szTemp, "%f", fValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

const String& String::operator = ( double dValue )
{
	char szTemp[128];
	sprintf( szTemp, "%f", dValue );
	*this = szTemp;
	_bIsNumericData = true;

	return *this;
}

void String::translateVars( const VariableList& varS )
{
	char* pszStart = strstr( _pszData, "$(" );
	char* pszEnd;
	char* pszEndT;
//	int nLen;
	String strTemp;
	String strName;
	const Variable* pvar;

	while( pszStart != NULL )
	{
		pszEnd = strchr( pszStart, ')' );

		strName = pszStart + 2;
		pszEndT = (char*)strchr( strName, ')' );
		if( pszEndT != NULL )
			*pszEndT = '\0';

		pvar = varS.find( strName );
		if( pvar == NULL )
			throw StringException( SE_NOSUCHVARIABLE );

		strTemp = mid( 0, (unsigned long)pszStart - (unsigned long)_pszData );
		strTemp += pvar->value();
		strTemp += pszEnd + 1;

		*this = strTemp;

		pszStart = strstr( _pszData, "$(" );
	}
}

void String::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl << ( ( _pszData == NULL ) ? "(null)" : _pszData ) << std::endl;
}

void String::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n" << ( ( _pszData == NULL ) ? "(null)" : _pszData ) << "\n";
}
