/*
	$Id: String.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __STRING_HPP__
#define __STRING_HPP__

#define RSTRING_DEFINED
#define STRING_NOTFOUND	(unsigned long)-1

#include <librfc/RFCBase.hpp>
#include <ostream>
#include <librfc/export.h>

BEGIN_RFC_NAMESPACE
class VariableList;
class Socket;

class RFC_Export String : public RFCBase
{
public:
//	typedef enum { null = 0 } StringNull;

private:
	char* _pszData;
	bool _bIsNull;
	unsigned long _ulDataLength; // including terminating null &c
	unsigned long _ulFixedLength; // excluding terminating null
	bool _bIsNumericData;
	char* _pszSQL;

public:
	// Construction
	String();
	String( const String& strString );
	String( const char* pszString, unsigned long ulFixedLength = 0 );
	String( unsigned long ulFixedLength );

	// Destruction
	virtual ~String();

	// Null Strings
	static const String null;
	bool isNull() const;

	// Assignment
	const String& copy( const String& strString, unsigned long ulLength = -1 );
	const String& copy( const char* pszString, unsigned long ulLength = -1 );
	const String& operator = ( const String& strString );
	const String& operator = ( const char* pszString );
	const String& operator += ( const String& strString );
	const String& operator += ( const char* pszString );
	const String& operator += ( char cCh );
	const String operator + ( const String& strString );
	const String operator + ( const char* pszString );
	const String operator + ( char cCh );

	// Comparison
	bool operator == ( const String& strString ) const;
	bool operator == ( const char* pszString ) const;
	bool operator != ( const String& strString ) const;
	bool operator != ( const char* pszString ) const;
	int compare( const String& strString, bool bCaseInsensitive = false ) const;
	int compare( const char* pszString, bool bCaseInsensitive = false ) const;
	int compareFor( const String& strString, unsigned long ulLength, bool bCaseInsensitive = false ) const;
	int compareFor( const char* pszString, unsigned long ulLength, bool bCaseInsensitive = false ) const;

	// Modification
	void fixLength( unsigned long ulFixedLength = 0 );
	unsigned long fixedLength() const;
	char* buffer( unsigned long ulLength = 0 );
	void release();
	void format( const char* pszFmt, ... );

	// Extraction
//	char operator[]( unsigned long ulPosition ) const;
	char at( unsigned long ulPosition ) const;

	// Access
	unsigned long length() const;
	operator const char*() const;

	// Location
	const char* find( char cCh, bool bReverse = false ) const;
	const char* find( const char* pszString ) const;
	unsigned long findIndex( char cCh, bool bReverse = false ) const;
	unsigned long findIndex( const char* pszString ) const;

	// Masking
	String createMask( const String& strSearch ) const;
	String mask( const String& strMask ) const;
	String replaceMask( const String& strMask, const String& strReplacementText ) const;

	// Trimming
	String trimRight() const;
	String trimLeft() const;
	String trim() const;

	// Escaping
	void unescape();
	void escape();
	void toURI();

	void fromURI();

	// Extraction
	String mid( unsigned long ulStart, unsigned long ulLength = (unsigned long)-1L ) const;
	String field( char cDelimiter, unsigned long ulIndex, char cFieldDelimiter = 0 ) const;

	// Outbound conversion
	operator long() const;
	operator unsigned long() const;
	operator double() const;
	const char* sql();

	// Inbound conversion
	const String& operator = ( signed char cValue );
	const String& operator = ( unsigned char byValue );
	const String& operator = ( signed short nValue );
	const String& operator = ( unsigned short uValue );
	const String& operator = ( signed long lvalue );
	const String& operator = ( unsigned long ulValue );
	const String& operator = ( float fValue );
	const String& operator = ( double dValue );

	// Odd stuff
	void translateVars( const VariableList& varS );

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__STRING_HPP__ */
