/*
	$Id: Time.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __TIME_HPP__
#define __TIME_HPP__

#include <librfc/String.hpp>
#include <librfc/FCConfig.hpp>
#include <librfc/export.h>
#include <librfc/RFCBase.hpp>

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export Time : public RFCBase
{
public:
	typedef enum { local, utc } Where;

protected:
	long _lHour;
	long _lMinute;
	double _dSecond;
	String _strFormat;

public:
	// __ctors
	Time();
	Time( long lHour, long lMinute, long lSecond );
	Time( long lHour, long lMinute, double dSecond );
	Time( const String& strTime, const String& strFormat = GetFCCString( SE_DEFAULT_TIME_FORMAT ) );
	Time( const Time& time );
	virtual ~Time();

	// operators
	const Time& operator= ( const String& strTime );
	const Time& operator= ( const Time& time );

	// accessors
	long hour() const { return _lHour; }
	long minute() const { return _lMinute; }
	double second() const { return _dSecond; }

	// mutators
	void hour( long lHour );
	void minute( long lMinute );
	void second( long lSecond );
	void second( double dSecond );

	String format( const String& strFormat = GetFCCString( SE_DEFAULT_TIME_FORMAT ) ) const;

	static Time now( Where where );

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__TIME_HPP__ */
