/*
	$Id: Timestamp.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/Timestamp.hpp>
#include <librfc/DateTimeException.hpp>

#include <librfc/Socket.hpp>


USE_RFC_NAMESPACE

Timestamp::Timestamp()
{
}

Timestamp::Timestamp( long lYear, long lMonth, long lDay,
			long lHour, long lMinute, long lSecond )
			: Date( lYear, lMonth, lDay ),
			  Time( lHour, lMinute, lSecond )
{
}

Timestamp::Timestamp( long lYear, long lMonth, long lDay,
			long lHour, long lMinute, double dSecond )
			: Date( lYear, lMonth, lDay ),
			  Time( lHour, lMinute, dSecond )
{
}

Timestamp::Timestamp( const String& strTimestamp, const String& strFormat )
{
	Date date( strTimestamp, strFormat );
	Time time( strTimestamp, strFormat );

	*this = Timestamp( date, time );
}

Timestamp::Timestamp( const Timestamp& timestamp )
{
	_lYear = timestamp._lYear;
	_lMonth = timestamp._lMonth;
	_lDay = timestamp._lDay;
	_lHour = timestamp._lHour;
	_lMinute = timestamp._lMinute;
	_dSecond = timestamp._dSecond;
}

Timestamp::Timestamp( const Date& date, const Time& time )
			: Date( date ),
			  Time( time )
{
}

Timestamp::~Timestamp()
{
}

const Timestamp& Timestamp::operator= ( const String& strTimestamp )
{
	// TODO
	return *this;
}

const Timestamp& Timestamp::operator= ( const Timestamp& timestamp )
{
	if( &timestamp != this )
	{
		_lYear = timestamp._lYear;
		_lMonth = timestamp._lMonth;
		_lDay = timestamp._lDay;
		_lHour = timestamp._lHour;
		_lMinute = timestamp._lMinute;
		_dSecond = timestamp._dSecond;
	}
	return *this;
}

String Timestamp::format( const String& strFormat )  const
{
	long lYearLength = strFormat.mask( strFormat.createMask( "Y" ) ).trim().length();
	long lMonthLength = strFormat.mask( strFormat.createMask( "M" ) ).trim().length();
	long lDayLength = strFormat.mask( strFormat.createMask( "D" ) ).trim().length();
	long lHourLength = strFormat.mask( strFormat.createMask( "H" ) ).trim().length();
	long lMinuteLength = strFormat.mask( strFormat.createMask( "N" ) ).trim().length();
	long lSecondLength = strFormat.mask( strFormat.createMask( "S" ) ).trim().length();
	String strYear;
	String strMonth;
	String strDay;
	String strHour;
	String strMinute;
	String strSecond;

	if( lYearLength == 0 )
		;
	else if( lYearLength == 2 )
	{
		strYear.format( "%02i", _lYear % 100 );
	}
	else if( lYearLength == 4 )
	{
		strYear.format( "%04i", _lYear );
	}
	else
	{
		DateTimeException e( DTE_INVALID_FORMAT );
		throw e;
	}

	if( lMonthLength == 0 )
		;
	else if( lMonthLength == 1 )
	{
		strMonth.format( "%i", _lMonth );
	}
	else if( lMonthLength == 2 )
	{
		strMonth.format( "%02i", _lMonth );
	}
	else if( lMonthLength == 3 )
	{
		strMonth.format( "%s", (const char*)Date::shortMonth( _lMonth - 1 ) );
	}
	else if( lMonthLength == 4 )
	{
		strMonth.format( "%s", (const char*)Date::longMonth( _lMonth - 1 ) );
	}
	else
	{
		DateTimeException e( DTE_INVALID_FORMAT );
		throw e;
	}

	if( lDayLength == 0 )
		;
	else if( lDayLength == 1 )
	{
		strDay.format( "%i", _lDay );
	}
	else if( lDayLength == 2 )
	{
		strDay.format( "%02i", _lDay );
	}

	else if( lDayLength == 3 )
	{
		strDay.format( "%03i", dayOfYear() );
	}
	else
	{
		DateTimeException e( DTE_INVALID_FORMAT );
		throw e;
	}

	if( lHourLength == 0 )
		;
	else if( lHourLength == 1 )
	{
		strHour.format( "%i", _lHour );
	}
	else if( lHourLength == 2 )
	{
		strHour.format( "%02i", _lHour );
	}
	else
	{
		DateTimeException e( DTE_INVALID_FORMAT );
		throw e;
	}

	if( lMinuteLength == 0 )
		;
	else if( lMinuteLength == 1 )
	{
		strMinute.format( "%i", _lMinute );
	}
	else if( lMinuteLength == 2 )
	{
		strMinute.format( "%02i", _lMinute );
	}
	else
	{
		DateTimeException e( DTE_INVALID_FORMAT );
		throw e;
	}

	if( lSecondLength == 0 )
		;
	else if( lSecondLength == 1 )
	{
		strSecond.format( "%i", (long)_dSecond );
	}
	else if( lSecondLength == 2 )
	{
		strSecond.format( "%02i", (long)_dSecond );
	}
	else if( ( lSecondLength == 3 ) || ( lSecondLength > 8 ) )
	{
		DateTimeException e( DTE_INVALID_FORMAT );
		throw e;
	}
	else
	{
		String strSecondFormat;
		strSecondFormat.format( "%%0%i.%if", lSecondLength, lSecondLength - 3 );
		strSecond.format( strSecondFormat, _dSecond );
	}

	String strRtn( strFormat );

	if( strFormat.createMask( "Y" ).trim().length() )
	{
		strRtn = strRtn.replaceMask( strFormat.createMask( "Y" ), strYear );
	}
	if( strFormat.createMask( "M" ).trim().length() )
	{
		strRtn = strRtn.replaceMask( strFormat.createMask( "M" ), strMonth );
	}
	if( strFormat.createMask( "D" ).trim().length() )
	{
		strRtn = strRtn.replaceMask( strFormat.createMask( "D" ), strDay );
	}
	if( strFormat.createMask( "H" ).trim().length() )
	{
		strRtn = strRtn.replaceMask( strFormat.createMask( "H" ), strHour );
	}
	if( strFormat.createMask( "N" ).trim().length() )
	{
		strRtn = strRtn.replaceMask( strFormat.createMask( "N" ), strMinute );
	}
	if( strFormat.createMask( "S." ).trim().length() )
	{
		strRtn = strRtn.replaceMask( strFormat.createMask( "S." ), strSecond );
	}

	return strRtn;
}

Timestamp Timestamp::now( Time::Where where )
{
	Date date( Date::now() );
	Time time( Time::now( where ) );

	return Timestamp( date, time );
}

void Timestamp::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl
		<< (const char*)Date::format() << " "
		<< (const char*)Time::format()
		<< std::endl;
}

void Timestamp::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n"
		<< (const char*)Date::format() << " "
		<< (const char*)Time::format()
		<< "\n";
}
