/*
	$Id: Timestamp.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __TIMESTAMP_HPP__
#define __TIMESTAMP_HPP__

#include <librfc/Time.hpp>
#include <librfc/Date.hpp>

#include <librfc/export.h>

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export Timestamp : public Date, public Time
{
public:
	// __ctors
	Timestamp();
	Timestamp( long lYear, long lMonth, long lDay,
				long lHour, long lMinute, long lSecond );
	Timestamp( long lYear, long lMonth, long lDay,
				long lHour, long lMinute, double dSecond );
	Timestamp( const String& strTimestamp, const String& strFormat = GetFCCString( SE_DEFAULT_TIMESTAMP_FORMAT ) );
	Timestamp( const Timestamp& timestamp );
	Timestamp( const Date& date, const Time& time );
	virtual ~Timestamp();

	// operators
	const Timestamp& operator= ( const String& strTimestamp );
	const Timestamp& operator= ( const Timestamp& timestamp );

	String format( const String& strFormat = GetFCCString( SE_DEFAULT_TIMESTAMP_FORMAT ) ) const;

	static Timestamp now( Time::Where where );

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__TIMESTAMP_HPP__ */
