/*
	$Id: Variable.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/Variable.hpp>
#include <stdlib.h>

#include <librfc/Socket.hpp>


USE_RFC_NAMESPACE

Variable::Variable( const String& strName, const String& strValue /*= String()*/ )
	: _pnext( NULL ),
	_strName( strName ),
	_strValue( strValue )
{
}

Variable::Variable( const Variable& variable )
	: _pnext( NULL ),
	_strName( variable._strName ),
	_strValue( variable._strValue )
{
}

Variable::~Variable()
{
	if( _pnext != NULL )
		delete _pnext;
}

void Variable::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl
		<< "N=" << ( _strName.isNull() ? "(null)" : (const char*)_strName )
		<< " V=" << ( _strValue.isNull() ? "(null)" : (const char*)_strValue )
		<< std::endl;
}

void Variable::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n"
		<< "N=" << ( _strName.isNull() ? "(null)" : (const char*)_strName )
		<< " V=" << ( _strValue.isNull() ? "(null)" : (const char*)_strValue )
		<< "\n";
}





VariableList::VariableList()
	: _pfirst( NULL ),
	_plast( NULL )
{
}

VariableList::~VariableList()
{
	if( _pfirst != NULL )
		delete _pfirst;
}

void VariableList::add( const Variable& variable )
{
	Variable* pvariable = new Variable( variable );

	if( _pfirst == NULL )
		_pfirst = _plast = pvariable;
	else
		_plast = _plast->_pnext = pvariable;
}

const Variable* VariableList::find( const String& strName, const Variable* pstart /*= NULL*/ ) const
{
	Variable* pfound = NULL;
	Variable* pcurrent;

	if( pstart == NULL )
		pcurrent = _pfirst;
	else
		pcurrent = pstart->_pnext;

	while( ( pcurrent != NULL ) && ( pfound == NULL ) )
	{
		if( pcurrent->_strName == strName )
			pfound = pcurrent;
		else
			pcurrent = pcurrent->_pnext;
	}

	return pfound;
}

void VariableList::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl;
}

void VariableList::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n";
}
