/*
	$Id: Variable.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Variable_hpp__
#define __Variable_hpp__

#include <librfc/String.hpp>
#include <librfc/RFCBase.hpp>

#include <librfc/export.h>

BEGIN_RFC_NAMESPACE
class Socket;

class RFC_Export Variable : public RFCBase
{
private:
	Variable* _pnext;
	String	_strName;
	String	_strValue;

public:
	Variable( const String& strName, const String& strValue = String() );
	Variable( const Variable& variable );
	virtual ~Variable();

	const String& name() const { return _strName; }
	const String& value() const { return _strValue; }
	void value( const String& strValue ) { _strValue = strValue; }

	friend class VariableList;

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};

class RFC_Export VariableList : public RFCBase
{
private:
	Variable* _pfirst;
	Variable* _plast;

public:
	VariableList();
	virtual ~VariableList();

	void add( const Variable& variable );
	const Variable* find( const String& strName, const Variable* pstart = NULL ) const;

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__Variable_hpp__ */
