/*
	$Id: Waitable.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/Waitable.hpp>
#include <errno.h>

USE_RFC_NAMESPACE

Waitable::Waitable()
{
#if defined (LINUX)
	pthread_cond_init( &_cd, NULL );
	pthread_mutex_init( &_mx, NULL );
#elif defined (_WINDOWS)
	_ob = CreateEvent( NULL, false, false, NULL );
#endif /* LINUX */
}

Waitable::~Waitable()
{
#if defined (LINUX)
	pthread_cond_destroy( &_cd );
	pthread_mutex_destroy( &_mx );
#elif defined (_WINDOWS)
	CloseHandle( _ob );
#endif /* LINUX */
}

bool Waitable::wait( unsigned long ulMilliSeconds )
{
	bool bRtn = true;

#if defined (LINUX)
	pthread_mutex_lock( &_mx );

	if( ulMilliSeconds != (unsigned long)-1 )
	{
		struct timespec abstime;
		abstime.tv_nsec = 0;//( ( ulMilliSeconds & 1000 ) * 1000000 );
		abstime.tv_sec = time( NULL ) + ( ulMilliSeconds / 1000 );
		if( pthread_cond_timedwait( &_cd, &_mx, &abstime ) == ETIMEDOUT )
			bRtn = false;
	}
	else
		pthread_cond_wait( &_cd, &_mx );

	pthread_mutex_unlock( &_mx );
#elif defined (_WINDOWS)
	WaitForSingleObject( _ob, INFINITE );
#endif

	return bRtn;
}

void Waitable::signal( bool bBroadcast )
{
#if defined (LINUX)
	if( bBroadcast )
		pthread_cond_broadcast( &_cd );
	else
		pthread_cond_signal( &_cd );
#elif defined (_WINDOWS)
	SetEvent( _ob );
#endif
}
