/*
	$Id: XMLAttr.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/XMLAttr.hpp>
#include <librfc/XMLNode.hpp>

#include <string.h>
#include <stdio.h>

#include <librfc/Socket.hpp>


USE_RFC_NAMESPACE

XMLAttr::XMLAttr( XMLNode* pnode, const String& strName, const String& strValue )
	: _strName( strName ),
	_strValue( strValue ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL )
{
	if( pnode != NULL )
		pnode->addAttr( this );
}

XMLAttr::XMLAttr( XMLNode* pnode, const char*& pszStream )
	: _psiblingPrev( NULL ),
	_psiblingNext( NULL )
{
	const char* pszPos;

	if( pnode->type() == XMLNode::typeParameter )
	{
		pszPos = strchr( pszStream, ASC_GREATER ); // at equal sign
	}
	else
	{
		pszPos = strchr( pszStream, ASC_EQUAL ); // at equal sign
		if( *( pszPos + 1 ) == ASC_DQUOTE )
		pszPos = strchr( pszPos, ASC_DQUOTE ); // at open quote
		pszPos = strchr( ( pszPos + 1 ), ASC_DQUOTE ); // at close quote
		pszPos = pszPos + 1;
	}

	String strStream;
	String strValue;

	strStream.copy( pszStream, ( pszPos - pszStream ) );
	_strName = strStream.field( ASC_EQUAL, 0, ASC_DQUOTE );
	strValue = strStream.field( ASC_EQUAL, 1, ASC_DQUOTE );
	_strValue = XMLUtil::XMLDecode( strValue );

	pszStream = pszPos;

	if( pnode != NULL )
		pnode->addAttr( this );
}

XMLAttr::~XMLAttr()
{
}

const String& XMLAttr::name() const
{
	return _strName;
}

const String& XMLAttr::value() const
{
	return _strValue;
}

void XMLAttr::value( const String& strValue )
{
	_strValue = strValue;
}

const String XMLAttr::stream() const
{
	String strRtn;
	String strValue;

	strValue = XMLUtil::XMLEncode( _strValue );
//	strValue = _strValue;
	strRtn.buffer( _strName.length() + strValue.length() + 8 );
	strRtn.format( "%s=\"%s\"", (const char*)_strName, (const char*)strValue );

	return strRtn;
}

void XMLAttr::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << "\n"
		<< "Nm=" << ( _strName.isNull() ? "(null)" : (const char*)_strName )
		<< " Va=" << ( _strValue.isNull() ? "(null)" : (const char*)_strValue )
		<< std::endl;
}

void XMLAttr::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n"
		<< "Nm=" << ( _strName.isNull() ? "(null)" : (const char*)_strName )
		<< " Va=" << ( _strValue.isNull() ? "(null)" : (const char*)_strValue )
		<< "\n";
}
