/*
	$Id: XMLDoc.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __xmldoc_h__
#define __xmldoc_h__

//#define USE_LIBXML

#include <librfc/XMLNode.hpp>
#include <librfc/String.hpp>
#include <librfc/RFCBase.hpp>

#ifdef USE_LIBXML
#	include <libxml/tree.h>
#endif

BEGIN_RFC_NAMESPACE
class Socket;

extern const char* XMLVersion;

class RFC_Export XMLDoc : public RFCBase
{
private:
	String _strVersion;
	String _strEncoding;
	String _strDocType;
	XMLNode* _pnode;
	bool _bIsNull;
#ifdef USE_LIBXML
	void create_elements( xmlNode *p_libxml_parent, XMLNode *pparent );
#endif

public:
	XMLDoc();
	XMLDoc( const String& strRootNodeName, const String& strVersion, const String& strEncoding = "UTF-8" );
	XMLDoc( const String& strRootNodeName, bool bDocType, const String& strVersion, const String& strEncoding = "UTF-8" );
	XMLDoc( const String& strDocType, const String& strVersion, const String& strEncoding, bool bCreateRootNode );
#ifdef USE_LIBXML
	XMLDoc( const char* pszStream, int iSize );
#endif
	XMLDoc( const char* pszStream );

	XMLDoc( const XMLDoc& doc );
	virtual ~XMLDoc();

	const XMLDoc& operator=( const XMLDoc& doc );

	bool isNull() const;

	const XMLNode* rootNode() const;
	XMLNode* rootNode();
	void rootNode( const XMLNode& node );
	void rootNode( const String& strName, bool bDocType );

	String docType() const;
	void docType( const String& strDocType );
	String version() const;
	String encoding() const;

	String stream() const;
	const XMLDoc& fromStream( const char*& pszStream );

#ifdef USE_LIBXML
	// this method will use libxml to parse the stream
	const XMLDoc& fromStream( const char*& pszStream, int iSize );
#endif

	const XMLDoc& fromFile( const char *pszFileName );

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__xmldoc_h__ */
