/*
	$Id: XMLNode.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/XMLNode.hpp>

#include <string.h>
#include <stdio.h>
#include <typeinfo>

#include <librfc/Socket.hpp>

#include <iostream>

USE_RFC_NAMESPACE

XMLNode::XMLNode( XMLNode* pparent, const String& strName, const String& strValue )
	: _strName( strName ),
	_strValue( strValue ),
	_type( typeNormal ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL ),
	_pchildFirst( NULL ),
	_pchildLast( NULL ),
	_pattrFirst( NULL ),
	_pattrLast( NULL ),
	_pparent( pparent )
{
	if( pparent != NULL )
		_pparent->addNodeToParent( this );
}

XMLNode::XMLNode( XMLNode* pparent, const String& strNameSpace, const String& strName, const String& strValue )
	: _strNameSpace( strNameSpace ),
	_strName( strName ),
	_strValue( strValue ),
	_type( typeNormal ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL ),
	_pchildFirst( NULL ),
	_pchildLast( NULL ),
	_pattrFirst( NULL ),
	_pattrLast( NULL ),
	_pparent( pparent )
{
	if( pparent != NULL )
		_pparent->addNodeToParent( this );
}

XMLNode::XMLNode( XMLNode* pparent, const char*& pszStream )
	: _type( typeNormal ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL ),
	_pchildFirst( NULL ),
	_pchildLast( NULL ),
	_pattrFirst( NULL ),
	_pattrLast( NULL ),
	_pparent( pparent )
{
	fromStream( pszStream );

	if( pparent != NULL )
		_pparent->addNodeToParent( this );
}

XMLNode::XMLNode( XMLNode* pparent, const XMLNode& nodeSource )
	: _type( typeNormal ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL ),
	_pchildFirst( NULL ),
	_pchildLast( NULL ),
	_pattrFirst( NULL ),
	_pattrLast( NULL ),
	_pparent( pparent )
{
	_strNameSpace = nodeSource._strNameSpace;
	_strName = nodeSource._strName;
	_strValue = nodeSource._strValue;
	_type = nodeSource._type;

	const XMLNode* pnode = NULL;
	pnode = nodeSource.nextChild( pnode );
	while( pnode != NULL )
	{
		addNode( *pnode );
		pnode = nodeSource.nextChild( pnode );
	}

	XMLAttr* pattr = nodeSource._pattrFirst;
	while( pattr != NULL )
	{
		attribute( pattr->name(), pattr->value() );
		pattr = pattr->_psiblingNext;
	}

/*
	String strStream( nodeSource.stream() );
	const char* pszStream = strchr( strStream, ASC_LESS );
	fromStream( pszStream );
*/
	if( pparent != NULL )
		_pparent->addNodeToParent( this );
}

XMLNode::XMLNode( const String& strName )
	: _strName( strName ),
	_type( typeNormal ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL ),
	_pchildFirst( NULL ),
	_pchildLast( NULL ),
	_pattrFirst( NULL ),
	_pattrLast( NULL ),
	_pparent( NULL )
{
}

XMLNode::XMLNode( const String& strNameSpace, const String& strName )
	: _strNameSpace( strNameSpace ),
	_strName( strName ),
	_type( typeNormal ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL ),
	_pchildFirst( NULL ),
	_pchildLast( NULL ),
	_pattrFirst( NULL ),
	_pattrLast( NULL ),
	_pparent( NULL )
{
}

XMLNode::XMLNode( const XMLNode& nodeSource )
	: _type( typeNormal ),
	_psiblingPrev( NULL ),
	_psiblingNext( NULL ),
	_pchildFirst( NULL ),
	_pchildLast( NULL ),
	_pattrFirst( NULL ),
	_pattrLast( NULL ),
	_pparent( NULL )
{
	_strNameSpace = nodeSource._strNameSpace;
	_strName = nodeSource._strName;
	_strValue = nodeSource._strValue;
	_type = nodeSource._type;

	const XMLNode* pnode = NULL;
	pnode = nodeSource.nextChild( pnode );
	while( pnode != NULL )
	{
		addNode( *pnode );
		pnode = nodeSource.nextChild( pnode );
	}

	XMLAttr* pattr = nodeSource._pattrFirst;
	while( pattr != NULL )
	{
		attribute( pattr->name(), pattr->value() );
		pattr = pattr->_psiblingNext;
	}
}

XMLNode::~XMLNode()
{
	deleteContent();

	if( _psiblingPrev == NULL )
	{
		if( _pparent != NULL )
			_pparent->_pchildFirst = _psiblingNext;
	}
	else
		_psiblingPrev->_psiblingNext = _psiblingNext;

	if( _psiblingNext == NULL )
	{
		if( _pparent != NULL )
			_pparent->_pchildLast = _psiblingPrev;
	}
	else
		_psiblingNext->_psiblingPrev = _psiblingPrev;
}

const XMLNode* XMLNode::parent() const
{
	return _pparent;
}

XMLNode* XMLNode::parent()
{
	return _pparent;
}

XMLNode* XMLNode::addNode( const String& strName, const String& strValue )
{
	return new XMLNode( this, strName, strValue );
}

XMLNode* XMLNode::addNode( const String& strNameSpace, const String& strName, const String& strValue )
{
	return new XMLNode( this, strNameSpace, strName, strValue );
}

XMLNode* XMLNode::addNode( const char*& pszStream )
{
	return new XMLNode( this, pszStream );
}

XMLNode* XMLNode::addNode( const XMLNode& nodeSource )
{
	return new XMLNode( this, nodeSource );
}

XMLNode* XMLNode::insertNodeAfter( const String& strName, const String& strValue )
{
	XMLNode* pnode = new XMLNode( _pparent, strName, strValue );

	return pnode;		// Exit point

	pnode->_pparent = _pparent;
	pnode->_psiblingPrev = this;

	if( _psiblingNext == NULL )
	{
		_pparent->_pchildLast = pnode;
	}
	else
	{
		_psiblingNext->_psiblingPrev = pnode;
	}

	_psiblingNext = pnode;

	return pnode;
}

XMLNode* XMLNode::insertNodeAfter( const String& strNameSpace, const String& strName, const String& strValue )
{
	XMLNode* pnode = new XMLNode( _pparent, strNameSpace, strName, strValue );

	return pnode;		// Exit point

	pnode->_pparent = _pparent;
	pnode->_psiblingPrev = this;

	if( _psiblingNext == NULL )
	{
		_pparent->_pchildLast = pnode;
	}
	else
	{
		_psiblingNext->_psiblingPrev = pnode;
	}

	_psiblingNext = pnode;

	return pnode;
}

XMLNode* XMLNode::insertNodeAfter( const char*& pszStream )
{
	XMLNode* pnode = new XMLNode( _pparent, pszStream );

	return pnode;		// Exit point

	pnode->_pparent = _pparent;
	pnode->_psiblingPrev = this;

	if( _psiblingNext == NULL )
	{
		_pparent->_pchildLast = pnode;
	}
	else
	{
		_psiblingNext->_psiblingPrev = pnode;
	}

	_psiblingNext = pnode;

	return pnode;
}

XMLNode* XMLNode::insertNodeAfter( const XMLNode& nodeSource )
{
	XMLNode* pnode = new XMLNode( _pparent, nodeSource );

	return pnode;		// Exit point

	pnode->_pparent = _pparent;
	pnode->_psiblingPrev = this;

	if( _psiblingNext == NULL )
	{
		_pparent->_pchildLast = pnode;
	}
	else
	{
		_psiblingNext->_psiblingPrev = pnode;
	}

	_psiblingNext = pnode;

	return pnode;
}

void XMLNode::deleteContent()
{
	if( _pchildFirst != NULL )
	{
		XMLNode* pnode = _pchildFirst;
		XMLNode* pnodeNext;

		while( pnode != NULL )
		{
			pnodeNext = pnode->_psiblingNext;
			delete pnode;
			pnode = pnodeNext;
		}
	}

	if( _pattrFirst != NULL )
	{
		XMLAttr* pattr = _pattrFirst;
		XMLAttr* pattrNext;

		while( pattr != NULL )
		{
			pattrNext = pattr->_psiblingNext;
			delete pattr;
			pattr = pattrNext;
		}
	}

	_pchildFirst = NULL;
	_pchildLast = NULL;
	_pattrFirst = NULL;
	_pattrLast = NULL;
}

const XMLNode& XMLNode::operator= ( const XMLNode& nodeSource )
{
	if( &nodeSource == this )
		return *this;

	deleteContent();

	_strNameSpace = nodeSource._strNameSpace;
	_strName = nodeSource._strName;
	_strValue = nodeSource._strValue;
	_type = nodeSource._type;

	const XMLNode* pnode = NULL;
	pnode = nodeSource.nextChild( pnode );
	while( pnode != NULL )
	{
		addNode( *pnode );
		pnode = nodeSource.nextChild( pnode );
	}

	XMLAttr* pattr = nodeSource._pattrFirst;
	while( pattr != NULL )
	{
		attribute( pattr->name(), pattr->value() );
		pattr = pattr->_psiblingNext;
	}

/*
	String strStream( nodeSource.stream() );
	const char* pszStream = strchr( strStream, ASC_LESS );
	fromStream( pszStream );
*/

	return *this;
}

XMLNode::Type XMLNode::type() const
{
	return _type;
}

void XMLNode::type( Type type )
{
	_type = type;
}

const String XMLNode::nameSpace() const
{
	return _strNameSpace;
}

const String XMLNode::name() const
{
	return _strName;
}

const String XMLNode::value() const
{
	return _strValue;
}

void XMLNode::value( const String& strValue )
{
	_strValue = strValue;
}

XMLNode* XMLNode::findChild( const String& strName, const XMLNode* pcurrent )
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( pnode->_strName == strName )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

const XMLNode* XMLNode::findChild( const String& strName, const XMLNode* pcurrent ) const
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( pnode->_strName == strName )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

XMLNode* XMLNode::findChild( const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent )
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( ( pnode->_strName == strName ) &&
			( pnode->attribute( strAttributeName ) == strAttributeValue ) )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

const XMLNode* XMLNode::findChild( const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent ) const
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( ( pnode->_strName == strName ) &&
			( pnode->attribute( strAttributeName ) == strAttributeValue ) )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

XMLNode* XMLNode::findChild( const String& strNameSpace, const String& strName, const XMLNode* pcurrent )
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( ( pnode->_strName == strName ) &&
			( pnode->_strNameSpace == strNameSpace ) )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

const XMLNode* XMLNode::findChild( const String& strNameSpace, const String& strName, const XMLNode* pcurrent ) const
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( ( pnode->_strName == strName ) &&
			( pnode->_strNameSpace == strNameSpace ) )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

XMLNode* XMLNode::findChild( const String& strNameSpace, const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent )
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( ( pnode->_strName == strName ) &&
			( pnode->_strNameSpace == strNameSpace ) &&
			( pnode->attribute( strAttributeName ) == strAttributeValue ) )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

const XMLNode* XMLNode::findChild( const String& strNameSpace, const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent ) const
{
	XMLNode* pnode;
	if( pcurrent == NULL )
		pnode = _pchildFirst;
	else
		pnode = pcurrent->_psiblingNext;
	XMLNode* pfound = NULL;

	while( ( pnode != NULL ) && ( pfound == NULL ) )
	{
		if( ( pnode->_strName == strName ) &&
			( pnode->_strNameSpace == strNameSpace ) &&
			( pnode->attribute( strAttributeName ) == strAttributeValue ) )
			pfound = pnode;
		else
			pnode = pnode->_psiblingNext;
	}

	return pfound;
}

XMLNode* XMLNode::nextChild( const XMLNode* pcurrent )
{
	if( pcurrent == NULL )
		return _pchildFirst;
	else
		return pcurrent->_psiblingNext;
}

const XMLNode* XMLNode::nextChild( const XMLNode* pcurrent ) const
{
	if( pcurrent == NULL )
		return _pchildFirst;
	else
		return pcurrent->_psiblingNext;
}

void XMLNode::findNodeS( XMLNodeList& list, const String& strName )
{
	// This function is used to search an entire document for all occurrences of
	// a given node.
	// It iterates through its children using nextChild.
	// For each child (match or no), it calls itself recursively to find other
	// children below the currently searched child
	// Then, if the child currently being tested matches, it too is added to the
	// list.
	// In this way, matching children further down the tree are added before
	// their parents.
	// The logic behind this is that in a serch and replace situation, if the
	// parent node were found first, all its children would be invalidated
	// before the parent and the program would crash when a no-longer existing
	// child was replaced.
	// BOTTOM UP, NOT TOP DOWN !

	XMLNode* pnodeSearch = nextChild();
	while( pnodeSearch != NULL )
	{
		pnodeSearch->findNodeS( list, strName );
		if( pnodeSearch->name() == strName )
			list.add( pnodeSearch );
		pnodeSearch = nextChild( pnodeSearch );
	}
}

void XMLNode::findNodeS( XMLNodeList& list, const String& strName, const String& strAttributeName, const String& strAttributeValue )
{
	// See above function for explanation

	XMLNode* pnodeSearch = nextChild();
	while( pnodeSearch != NULL )
	{
		pnodeSearch->findNodeS( list, strName, strAttributeName, strAttributeValue );
		if( ( pnodeSearch->name() == strName ) &&
			( pnodeSearch->attribute( strAttributeName ) == strAttributeValue ) )
			list.add( pnodeSearch );
		pnodeSearch = nextChild( pnodeSearch );
	}
}

void XMLNode::findNodeS( XMLNodeList& list, const String& strNameSpace, const String& strName )
{
	// See above function for explanation

	XMLNode* pnodeSearch = nextChild();
	while( pnodeSearch != NULL )
	{
		pnodeSearch->findNodeS( list, strNameSpace, strName );
		if( ( pnodeSearch->name() == strName ) &&
			( pnodeSearch->nameSpace() == strNameSpace ) )
			list.add( pnodeSearch );
		pnodeSearch = nextChild( pnodeSearch );
	}
}

void XMLNode::findNodeS( XMLNodeList& list, const String& strNameSpace, const String& strName, const String& strAttributeName, const String& strAttributeValue )
{
	// See above function for explanation

	XMLNode* pnodeSearch = nextChild();
	while( pnodeSearch != NULL )
	{
		pnodeSearch->findNodeS( list, strNameSpace, strName, strAttributeName, strAttributeValue );
		if( ( pnodeSearch->name() == strName ) &&
			( pnodeSearch->nameSpace() == strNameSpace ) &&
			( pnodeSearch->attribute( strAttributeName ) == strAttributeValue ) )
			list.add( pnodeSearch );
		pnodeSearch = nextChild( pnodeSearch );
	}
}

const String XMLNode::attribute( const String& strName ) const
{
	const XMLAttr* pattr = findAttribute( strName );
	if( pattr == NULL )
		return String::null;

	return pattr->value();
}

void XMLNode::attribute( const String& strName, const String& strValue )
{
	XMLAttr* pattr = findAttribute( strName );

	if( pattr == NULL )
		new XMLAttr( this, strName, strValue );
	else
		pattr->value( strValue );
}

XMLAttr* XMLNode::attribute( XMLAttr* pcurrent ) const
{
	if( pcurrent == NULL )
		return _pattrFirst;
	else
		return pcurrent->_psiblingNext;

}

const String XMLNode::stream( bool bIndent /*= true*/ ) const
{
	String strRtn;

	long lLevel = level();
	long lTab;
	if( bIndent )
	{
		for( lTab = 0; lTab < lLevel; lTab++ )
			strRtn += ASC_TAB;
	}

	if( _type == typeCommentedOut )
	{
		strRtn += _strValue;
		strRtn += ASC_NEWLINE;
	}
	else
	{
		strRtn += ASC_LESS;
		if( _strNameSpace.length() )
		{
			strRtn += _strNameSpace;
			strRtn += ":";
		}
		strRtn += _strName;

		XMLAttr* pattr = _pattrFirst;
		while( pattr != NULL )
		{
			strRtn += ASC_SPACE;
			strRtn += pattr->stream();
			pattr = pattr->_psiblingNext;
		}

		if( _pchildFirst != NULL )
		{
			strRtn += ASC_GREATER;
			strRtn += ASC_NEWLINE;
			XMLNode* pnode = _pchildFirst;
			while( pnode != NULL )
			{
				strRtn += pnode->stream();
				pnode = pnode->_psiblingNext;
			}
			for( lTab = 0; lTab < lLevel; lTab++ )
				strRtn += ASC_TAB;
			strRtn += ASC_LESS;
			strRtn += ASC_SLASH;
			if( _strNameSpace.length() )
			{
				strRtn += _strNameSpace;
				strRtn += ":";
			}
			strRtn += _strName;
			strRtn += ASC_GREATER;
			strRtn += ASC_NEWLINE;
		}
		else if( _strValue.length() )
		{
			if( !_strName.compareFor( XML_COMMENTSTART, 3 ) )
			{
				strRtn += _strValue;
				strRtn += ASC_NEWLINE;
			}
			else
			{
				strRtn += ASC_GREATER;
				strRtn += XMLUtil::XMLEncode( _strValue );
				strRtn += ASC_LESS;
				strRtn += ASC_SLASH;
				if( _strNameSpace.length() )
				{
					strRtn += _strNameSpace;
					strRtn += ":";
				}
				strRtn += _strName;
				strRtn += ASC_GREATER;
				strRtn += ASC_NEWLINE;
			}
		}
		else
		{
			strRtn += ASC_SLASH;
			strRtn += ASC_GREATER;
			strRtn += ASC_NEWLINE;
		}
	}

	return strRtn;
}

long XMLNode::level() const
{
	if( _pparent == NULL )
		return 0;
	else
		return _pparent->level() + 1;
}

XMLAttr* XMLNode::findAttribute( const String& strName )
{
	XMLAttr* pattr = _pattrFirst;
	XMLAttr* pfound = NULL;

	while( ( pattr != NULL ) && ( pfound == NULL ) )
	{
		if( pattr->name() == strName )
			pfound = pattr;
		else
			pattr = pattr->_psiblingNext;
	}

	return pfound;
}

const XMLAttr* XMLNode::findAttribute( const String& strName ) const
{
	XMLAttr* pattr = _pattrFirst;
	XMLAttr* pfound = NULL;

	while( ( pattr != NULL ) && ( pfound == NULL ) )
	{
		if( pattr->name() == strName )
			pfound = pattr;
		else
			pattr = pattr->_psiblingNext;
	}

	return pfound;
}

void XMLNode::addNodeToParent( XMLNode* pnode )
{
	if( _pchildFirst == NULL )
	{
		_pchildFirst = _pchildLast = pnode;
	}
	else
	{
		pnode->_psiblingPrev = _pchildLast;
		_pchildLast->_psiblingNext = pnode;
		_pchildLast = pnode;
	}
}

void XMLNode::addAttr( XMLAttr* pattr )
{
	if( _pattrFirst == NULL )
	{
		_pattrFirst = _pattrLast = pattr;
	}
	else
	{
		pattr->_psiblingPrev = _pattrLast;
		_pattrLast = _pattrLast->_psiblingNext = pattr;
	}
}

void XMLNode::fromStream( const char*& pszStream )
{
	const char* pszOpenTagEnd = strchr( pszStream, ASC_GREATER );
	const char* pszFirstSpace = strchr( pszStream, ASC_SPACE );
	const char* pszNextTag;

	if( *pszStream != ASC_LESS )
		throw XMLException( XE_N_NOTANODE );

	switch( *( pszStream + 1 ) )
	{
		case ASC_QUESTION:
			_strName = _strName.mid( 1 );
			_type = typeComment;
			break;

		case ASC_BANG:
			_strName = _strName.mid( 1 );
			_type = typeParameter;
			break;

		default:
			_type = typeNormal;
	}

	if( ( pszFirstSpace != NULL ) && ( pszFirstSpace < pszOpenTagEnd ) )
	{
		// There is a space within the node so there are attributes
		_strName.copy( ( pszStream + 1 ), ( pszFirstSpace - ( pszStream + 1 ) ) );
		pszStream += ( _strName.length() + 1 );
		size_t ls;
//		ls = strspn( pszStream, " \t\n\r" );
//		if( ls )
//		{
//			pszStream += ls;
//		}
//		else
//		{
//			char* psz = strstr( pszStream, "?>" );
//			if( psz!= NULL )
//				pszStream = psz;
//		}
		ls = strspn( pszStream, " \t\n\r?" );
		pszStream = pszStream + ls;

		if( !_strName.compare( "!DOCTYPE", true ) )
		{
			// this is the DTD, so just copy the whole damn lot
			char* pszEnd = (char*)strchr( pszStream, ASC_GREATER );
			if( pszEnd == NULL )
				throw XMLException( XE_N_NOTANODE );
			String strDocType;
			strDocType.copy( pszStream, pszEnd - pszStream );
			new XMLAttr( this, strDocType, String::null );
			pszStream = pszEnd;
		}
		else if( !_strName.compareFor( XML_COMMENTSTART, 3, true ) )
		{
			// this is a comment
			_type = typeCommentedOut;
			char* pszEnd = (char*)strstr( pszStream, XML_COMMENTEND );
			if( pszEnd == NULL )
				throw XMLException( XE_N_NOTANODE );
			_strValue.copy( pszStream - _strName.length() - 2, ( pszEnd - pszStream ) + _strName.length() + 5 );
			pszStream = pszEnd + 3;
		}
		else
		{
			bool bContinue( true );
			while( bContinue )
			{
				new XMLAttr( this, pszStream );
//				ls = strspn( pszStream, " \t\n\r" );
//				if( ls )
//				{
//					pszStream += ls;
//				}
//				else
//				{
//					char* psz = strstr( pszStream, "?>" );
//					if( psz!= NULL )
//						pszStream = psz;
//				}
				ls = strspn( pszStream, " \t\n\r?" );
				pszStream = pszStream + ls;
				if( ( *pszStream == ASC_SLASH ) || ( *pszStream == ASC_GREATER ) )
					bContinue = false;
			}
		}
	}
	else
	{
		_strName.copy( ( pszStream + 1 ), ( pszOpenTagEnd - ( pszStream + 1 ) ) );
		pszStream = strchr( pszStream, ASC_GREATER );
	}

	switch( _type )
	{
		case typeComment:
			_strName = _strName.mid( 1 );
			break;

		case typeParameter:
			_strName = _strName.mid( 1 );
			break;

		case typeNormal:
		case typeCommentedOut:
			break;
	}

	if( pszOpenTagEnd == NULL )
		throw XMLException( XE_N_MALFORMEDNODE );

	// If the character before the > is a /, this is an empty node
	if( *( pszOpenTagEnd - 1 ) == ASC_SLASH )
	{
		// This is an empty node
		pszStream = pszOpenTagEnd + 1;
		if( _pattrFirst == NULL )
			_strName = _strName.mid( 0, ( _strName.length() - 1 ) );
	}
	else if( _type == typeNormal )
	{
		char c = *( pszStream + 1 + strspn( pszStream + 1, " \t\n\r" ) );
		if( c == ASC_LESS )
		{
			// We have child nodes
			bool bContinue( true );
			do
			{
				pszNextTag = strchr( pszStream, ASC_LESS );
				if( pszNextTag != NULL )
				{
					if( *( pszNextTag + 1 ) == ASC_SLASH )
					{
						// This is the closing tag for this node
						pszNextTag = strchr( pszNextTag, ASC_GREATER );
						pszStream = pszNextTag + 1;
						bContinue = false;
					}
					else
					{
						// This is the opening tag of a child node
						pszStream = pszNextTag;
						new XMLNode( this, pszStream );
					}
				}
				else
					bContinue = false;
			} while( bContinue );
		}
		else
		{
			pszNextTag = strchr( pszStream, ASC_LESS );
			String strValue;
			strValue.copy( ( pszStream + 1 ), ( pszNextTag - ( pszStream + 1 ) ) );
			_strValue = XMLUtil::XMLDecode( strValue );
			pszStream = pszNextTag;
			pszStream = strchr( ( pszStream + 1 ), ASC_LESS );
		}
	}
	else
	{
		pszStream = strchr( pszStream, ASC_LESS );
	}

	const char* pszColon = _strName.find( ':' );

	if( pszColon != NULL )
	{
		_strNameSpace = _strName.field( ':', 0 );
		_strName = _strName.field( ':', 1 );
	}
}

void XMLNode::dump( std::ostream& os )
{
	os << "@ " << typeid( this ).name() << std::endl
		<< "Ns=" << ( _strNameSpace.isNull() ? "(null)" : (const char*)_strNameSpace )
		<< " Nm=" << ( _strName.isNull() ? "(null)" : (const char*)_strName )
		<< " Va=" << ( _strValue.isNull() ? "(null)" : (const char*)_strValue )
		<< std::endl;
}

void XMLNode::dump( Socket& skt )
{
	skt << "@ " << typeid( this ).name() << "\n"
		<< "Ns=" << ( _strNameSpace.isNull() ? "(null)" : (const char*)_strNameSpace )
		<< " Nm=" << ( _strName.isNull() ? "(null)" : (const char*)_strName )
		<< " Va=" << ( _strValue.isNull() ? "(null)" : (const char*)_strValue )
		<< "\n";
}










#if( 0 )




XMLNode::XMLNode( XMLNode* pparent, const String& strName, const String& strValue )
		: _strName( strName ),
		_strValue( strValue ),
		_cType( TYPE_NORMAL ),
		_psiblingPrev( NULL ),
		_psiblingNext( NULL ),
		_pchildFirst( NULL ),
		_pchildLast( NULL ),
		_pattrFirst( NULL ),
		_pattrLast( NULL ),
		_pparent( pparent )
{
	if( _pparent == NULL )
		_lLevel = 0;
	else
		_lLevel = _pparent->_lLevel + 1;

	if( strName.isNull() )
	{
		throw XMLException( XE_N_NOANONNODES );
	}

	if( _pparent != NULL )
		_pparent->addNode( this );
}

XMLNode::XMLNode( XMLNode* pparent, const String& strStream )
		: _cType( TYPE_NORMAL ),
		_psiblingPrev( NULL ),
		_psiblingNext( NULL ),
		_pchildFirst( NULL ),
		_pchildLast( NULL ),
		_pattrFirst( NULL ),
		_pattrLast( NULL ),
		_pparent( pparent )
{
	if( _pparent == NULL )
		_lLevel = 0;
	else
		_lLevel = _pparent->_lLevel + 1;

	bool bEmpty( false );

	const char* pszStream = strStream;
	while(( *pszStream == ' ' ) || ( *pszStream == '\t' ) )
		pszStream++;

	if( *pszStream != ASC_LESS )
	{
		throw XMLException( XE_N_NOTANODE );
	}

	char* pszEndName;

	switch( *( pszStream + 1 ) )
	{
		case ASC_QUESTION:
			pszEndName = strpbrk( pszStream + 3, NODE_NAMEEND );
			if( pszEndName == NULL )
			{
				throw XMLException( XE_N_INVALIDNODE );
			}
/*
			_pszName = new char[( pszEndName - pszStream )];
			strncpy( _pszName, pszStream + 2, ( pszEndName - pszStream - 2 ) );
			_pszName[( ( pszEndName - pszStream ) - 2 )] = ASC_NUL;
*/
			_strName.copy( pszStream + 2, ( pszEndName - pszStream - 2 ) );
			bEmpty = true;
			_cType = TYPE_COMMENT;
			break;

		case ASC_BANG:
			pszEndName = strpbrk( pszStream + 2, NODE_NAMEEND );
			if( pszEndName == NULL )
			{
				throw XMLException( XE_N_INVALIDNODE );
			}
/*
			_pszName = new char[( pszEndName - pszStream )];
			strncpy( _pszName, pszStream + 2, ( pszEndName - pszStream ) );
			_pszName[( ( pszEndName - pszStream ) - 2 )] = ASC_NUL;
*/
			_strName.copy( pszStream + 2, ( pszEndName - pszStream - 2 ) );
			bEmpty = true;
			_cType = TYPE_PARAMETER;
			break;

		default:
			pszEndName = strpbrk( pszStream + 1, NODE_NAMEEND );
			if( pszEndName == NULL )
			{
				throw XMLException( XE_N_INVALIDNODE );
			}
/*
			_pszName = new char[( pszEndName - pszStream )];
			strncpy( _pszName, pszStream + 1, ( pszEndName - pszStream ) );
			_pszName[( ( pszEndName - pszStream ) - 1 )] = ASC_NUL;
*/
			_strName.copy( pszStream + 1, ( pszEndName - pszStream - 1 ) );
	}

	switch( *pszEndName )
	{
		case ASC_SLASH:		// Empty node with no attributes
			break;

		case ASC_SPACE:		// Node with attributes
		{
			char* pszPos = pszEndName;
			char* pszEnd;
			bool bInQuote = false;
			bool bContinue = true;
			while( bContinue )
			{
				while( *pszPos == ASC_SPACE )
				{
					pszPos++;
				}

				pszEnd = pszPos;

				while( ( ( *pszEnd != ASC_SPACE ) &&
					( *pszEnd != ASC_SLASH ) &&
					( *pszEnd != ASC_QUESTION ) &&
					( *pszEnd != ASC_GREATER ) ) ||
					( bInQuote ) )
				{
					if( *pszEnd == ASC_DQUOTE )
						bInQuote = !bInQuote;
					pszEnd++;
				}

				char* pszTemp = new char[( pszEnd - pszPos + 1 )];
				strncpy( pszTemp, pszPos, pszEnd - pszPos );
				pszTemp[( pszEnd - pszPos )] = ASC_NUL;

				try
				{
					XMLAttr* pattr = new XMLAttr( pszTemp );
					if( _pattrFirst == NULL )
					{
						_pattrFirst = _pattrLast = pattr;
					}
					else
					{
						pattr->_psiblingPrev = _pattrLast;
						_pattrLast->_psiblingNext = pattr;
						_pattrLast = pattr;
					}
				}
				catch( const XMLException& )
				{
					delete pszTemp;
					throw;
				}
				delete pszTemp;

				if( ( *pszEnd == ASC_SLASH ) ||
					( *pszEnd == ASC_GREATER ) ||
					( *pszEnd == ASC_QUESTION ) )
				{
					bContinue = false;
					if( *pszEnd == ASC_SLASH )
						bEmpty = true;
				}
				else
				{
					pszPos = pszEnd++;
				}
			}
		}
		// Fall through
		case ASC_GREATER:	// Node without attributes
		{
			if( !bEmpty )
			{
				char* pszStart = strchr( pszStream, ASC_GREATER );
				char* pszEnd = strrchr( pszStream, ASC_LESS );
				if( ( pszStart == NULL ) ||
					( pszEnd == NULL ) ||
					( pszEnd <= pszStart ) )
				{
					throw XMLException( XE_N_MALFORMEDNODE );
				}

				char* pszTemp = new char[( pszEnd - pszStart )];
				char* pszTempNode = new char[( pszEnd - pszStart )];
				strncpy( pszTemp, pszStart + 1, pszEnd - pszStart );
				pszTemp[( pszEnd - pszStart - 1 )] = ASC_NUL;
				if( strchr( pszTemp, ASC_LESS ) )
				{
					try
					{
						const char* pszPos = pszTemp;
						while( strchr( pszPos, ASC_LESS ) )
						{
//							pszPos = XMLUtil::getNode( pszTempNode, pszPos );
							/*XMLNode* pnode =*/ new XMLNode( this, pszTempNode );
						}
					}
					catch( const XMLException& )
					{
						delete pszTemp;
						delete pszTempNode;
						throw XMLException( XE_N_INVALIDDATA );
					}
				}
				else
				{
//					XMLUtil::XMLDecode( _strValue.buffer( XMLUtil::XMLDecodedLen( pszTemp ) + 1 ), pszTemp );
				}

				delete pszTemp;
				delete pszTempNode;
			}
		}
	}

	if( _pparent != NULL )
		_pparent->addNode( this );
}

XMLNode::~XMLNode()
{
	if( _pchildFirst != NULL )
	{
		XMLNode* pnode = _pchildFirst;
		XMLNode* pnodeNext;

		while( pnode != NULL )
		{
			pnodeNext = pnode->_psiblingNext;
			delete pnode;
			pnode = pnodeNext;
		}
	}

	if( _pattrFirst != NULL )
	{
		XMLAttr* pattr = _pattrFirst;
		XMLAttr* pattrNext;

		while( pattr != NULL )
		{
			pattrNext = pattr->_psiblingNext;
			delete pattr;
			pattr = pattrNext;
		}
	}

	if( _psiblingPrev == NULL )
	{
		if( _pparent != NULL )
			_pparent->_pchildFirst = _psiblingNext;
	}
	else
		_psiblingPrev->_psiblingNext = _psiblingNext;

	if( _psiblingNext == NULL )
	{
		if( _pparent != NULL )
			_pparent->_pchildLast = _psiblingPrev;
	}
	else
		_psiblingNext->_psiblingPrev = _psiblingPrev;
}

char XMLNode::type() const
{
	return _cType;
}

void XMLNode::type( char cType )
{
	_cType = cType;
}

String XMLNode::name() const
{
	return _strName;
}

String XMLNode::value() const
{
	return _strValue;
}

void XMLNode::value( const String& strValue )
{
	if( _pchildFirst != NULL )
	{
		throw XMLException( XE_N_NODEHASCHILDREN );
	}

	_strValue = strValue;
}


String XMLNode::stream()
{
	size_t lLenBuff = streamLen();
	size_t lLenData = 0;
	size_t lLenAttr = 0;
	size_t lLenNode = 0;

	char* pszPos = _strNode.buffer( lLenBuff + 1 );
//	char* pszPos = pszNode;

	for( long lTab = 0; lTab < _lLevel; lTab++ )
	{
		*pszPos++;
		_strNode += '\t';
	}

	size_t lLenTag = _strName.length() + 4;
	char* pszTag = new char[( lLenTag + 1 )];

	String strTemp;
	strTemp.format( "<%s", (const char*)_strName );
	strcpy( pszTag, strTemp );

	strcpy( pszPos, pszTag );
	pszPos += strlen( pszTag );

	XMLAttr* pattr = _pattrFirst;
	while( pattr != NULL )
	{
//		lLenAttr = pattr->streamLen();
		*pszPos++ = ASC_SPACE;
		strcpy( pszPos, pattr->stream() );
		pszPos += lLenAttr;
		pattr = pattr->_psiblingNext;
	}

	if( ( _pchildFirst == NULL ) &&
		( !_strValue.length() ) )
	{
		strcpy( pszPos, "/>" );
		pszPos += 2;
	}
	else
	{
		strcpy( pszPos, ">" );
		pszPos++;

		XMLNode* pnode = _pchildFirst;
		if( pnode != NULL )
		{
//			*pszPos++ = '\n';
			while( pnode != NULL )
			{
				lLenNode = pnode->streamLen();
				String strChild = pnode->stream();
//				lLenNode = XMLUtil::XMLEncodedLen( strChild );
				strcpy( pszPos, strChild );
				while( *++pszPos );
//				pszPos += lLenNode;
				pnode->release();
				pnode = pnode->_psiblingNext;
			}
		}
		else
		{
			if( !_strValue.isNull() )
			{
//				lLenData = XMLUtil::XMLEncodedLen( _strValue );
//				XMLUtil::XMLEncode( pszPos, _strValue );
				pszPos += lLenData;
			}
		}
		String strTemp;
		strTemp.format( "</%s>", (const char*)_strName );
		strcpy( pszTag, strTemp );
		strcpy( pszPos, pszTag );
		pszPos += strlen( pszTag );
	}

	strcat( pszPos++, "\n" );

	*pszPos = ASC_NUL;

	delete pszTag;

//	printf( "B=%i L=%i\n", lLenBuff, strlen( _pszNode ) );
//	printf( "%s\n", _pszNode );
	return _strNode;
}

size_t XMLNode::streamLen() const
{
	/*
		The length of a node is calculated as:
		The sum of the lengths of all chidl nodes -
				streamLen is called recursively
		OR the length of the node's data after encoding
		PLUS the sum of the encoded lengths of all attributes
		PLUS one space for each attribute
		IF the data portion is empty
			PLUS the length of the name
			PLUS two chracters for the tag brackets
			PLUS one character for the terminating slash
		OTHERWISE
			PLUS twice the length of the name (start/end tags)
			PLUS four characters for the tag brackets
			PLUS one character for the terminating slash

		Finally, add level (for tabs) + 1 for carriage return.
	*/

	size_t lLen = _lLevel + 1;

	XMLNode* pnode = _pchildFirst;
	if( pnode != NULL )
	{
		while( pnode != NULL )
		{
			lLen += pnode->streamLen();
			pnode = pnode->_psiblingNext;
		}
	}
	else
	{
		if( !_strValue.isNull() )
		{
//			lLen += XMLUtil::XMLEncodedLen( _strValue );
		}
	}

	if( lLen )
	{
		// Node is not empty
		lLen += 2 * _strName.length();
		lLen += 5;
	}
	else
	{
		// Node is empty
		lLen += _strName.length();
		lLen += 3;
	}

	XMLAttr* pattr = _pattrFirst;
	while( pattr != NULL )
	{
//		lLen += pattr->streamLen();
		lLen += 1;
		pattr = pattr->_psiblingNext;
	}

	return lLen;
}

void XMLNode::release()
{
	_strNode = String::null;

	XMLNode* pnode = _pchildFirst;
	if( pnode != NULL )
	{
		while( pnode != NULL )
		{
			pnode->release();
			pnode = pnode->_psiblingNext;
		}
	}

	XMLAttr* pattr = _pattrFirst;
	if( pattr != NULL )
	{
		while( pattr != NULL )
		{
//			pattr->release();
			pattr = pattr->_psiblingNext;
		}
	}
}


#endif
