/*
	$Id: XMLNode.hpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __xmlnode_h__
#define __xmlnode_h__

#include <librfc/XMLAttr.hpp>
#include <librfc/String.hpp>
#include <librfc/RFCBase.hpp>


BEGIN_RFC_NAMESPACE
class XMLNode;
class Socket;

//These two classes are used for searching documents for nodes
class RFC_Export XMLSNode
{
private:
	XMLNode* _pnode;
	XMLSNode* _pnext;

	XMLSNode( XMLNode* pnode );

	friend class XMLNodeList;

public:
	XMLSNode();
	virtual ~XMLSNode();
};

class RFC_Export XMLNodeList
{
private:
	XMLSNode* _pfirst;
	XMLSNode* _plast;

public:
	XMLNodeList();
	virtual ~XMLNodeList();

	void clear();
	XMLNode* next( XMLSNode*& piterator );

protected:
	void add( XMLNode* pnode );

	friend class XMLNode;
};

class RFC_Export XMLNode : public RFCBase
{
public:
	typedef enum{ typeNormal, typeComment, typeParameter, typeCommentedOut }Type;

private:
	String _strNameSpace;
	String _strName;
	String _strValue;
	Type _type;
	XMLNode* _psiblingPrev;
	XMLNode* _psiblingNext;
	XMLNode* _pchildFirst;
	XMLNode* _pchildLast;
	XMLAttr* _pattrFirst;
	XMLAttr* _pattrLast;
	XMLNode* _pparent;

private:
	XMLNode(){}
	XMLNode( XMLNode* pparent, const String& strName, const String& strValue );
	XMLNode( XMLNode* pparent, const String& strNameSpace, const String& strName, const String& strValue );
	XMLNode( XMLNode* pparent, const char*& pszStream );
	XMLNode( XMLNode* pparent, const XMLNode& nodeSource );

public:
	XMLNode( const String& strNameSpace, const String& strName );
	XMLNode( const String& strName );
	XMLNode( const XMLNode& nodeSource );
	virtual ~XMLNode();

	const XMLNode* parent() const;
	XMLNode* parent();

	XMLNode* addNode( const String& strName, const String& strValue );
	XMLNode* addNode( const String& strNameSpace, const String& strName, const String& strValue );
	XMLNode* addNode( const char*& pszStream );
	XMLNode* addNode( const XMLNode& nodeSource );

	XMLNode* insertNodeAfter( const String& strName, const String& strValue );
	XMLNode* insertNodeAfter( const String& strNameSpace, const String& strName, const String& strValue );
	XMLNode* insertNodeAfter( const char*& pszStream );
	XMLNode* insertNodeAfter( const XMLNode& nodeSource );

	void deleteContent();
	const XMLNode& operator= ( const XMLNode& nodeSource );

	Type type() const;
	void type( Type type );

	const String nameSpace() const;
	const String name() const;
	const String value() const;
	void value( const String& strValue );

	XMLNode* findChild( const String& strName, const XMLNode* pcurrent = NULL );
	const XMLNode* findChild( const String& strName, const XMLNode* pcurrent = NULL ) const;
	XMLNode* findChild( const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent = NULL );
	const XMLNode* findChild( const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent = NULL ) const;
	XMLNode* findChild( const String& strNameSpace, const String& strName, const XMLNode* pcurrent = NULL );
	const XMLNode* findChild( const String& strNameSpace, const String& strName, const XMLNode* pcurrent = NULL ) const;
	XMLNode* findChild( const String& strNameSpace, const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent = NULL );
	const XMLNode* findChild( const String& strNameSpace, const String& strName, const String& strAttributeName, const String& strAttributeValue, const XMLNode* pcurrent = NULL ) const;

	XMLNode* nextChild( const XMLNode* pcurrent = NULL );
	const XMLNode* nextChild( const XMLNode* pcurrent = NULL ) const;

	void findNodeS( XMLNodeList& list, const String& strName );
	void findNodeS( XMLNodeList& list, const String& strName, const String& strAttributeName, const String& strAttributeValue );
	void findNodeS( XMLNodeList& list, const String& strNameSpace, const String& strName );
	void findNodeS( XMLNodeList& list, const String& strNameSpace, const String& strName, const String& strAttributeName, const String& strAttributeValue );

	const String attribute( const String& strName ) const;
	void attribute( const String& strName, const String& strValue );
	XMLAttr* attribute( XMLAttr* pcurrent = NULL ) const;

	const String stream( bool bIndent = true ) const;

	long level() const;
	void fromStream( const char*& pszStream );


protected:
	XMLAttr* findAttribute( const String& strName );
	const XMLAttr* findAttribute( const String& strName ) const;
	void addNodeToParent( XMLNode* pnode );
	void addAttr( XMLAttr* pattr );

	friend class XMLDoc;
	friend class XMLAttr;

protected:
	virtual void dump( std::ostream& os );
	virtual void dump( Socket& skt );
};
END_RFC_NAMESPACE

#endif /* !__xmlnode_h__ */
