/*
	$Id: XMLUtil.cpp 1205 2018-11-22 18:09:31Z john $

	This file is part of the Revelation Foundation Classes.

	The RFC is Copyright (c) 1989 - 1998, Revelation Software Ltd.
	The RFC is Copyright (c) 1998 - 2018, The 4c Company.

	The RFC is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	The RFC is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <librfc/XMLUtil.hpp>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if defined (_WINDOWS)
#	include <librfc/String.hpp>
#endif

typedef struct __tagXMLCodeMap
{
	char cChar;
	const char* pszCode;
	size_t lLen;
} XMLCodeMap;

/*
	Note: When adding codes to the map, be sure to add them
	to the CodedS array as well. Codes added to the map must
	appear before the terminating NULL entry.
*/
static XMLCodeMap map[] = {	{ '%', "&per;", 5 },
							{ '&', "&amp;", 5 },
							{ '>', "&gt;", 4 },
							{ '<', "&lt;", 4 },
							{ '/', "&slash;", 7 },
							{ '?', "&qm;", 4 },
							{ '!', "&em;", 4 },
							{ '-', "&dash;", 6 },
							{ '[', "&ob;", 4 },
							{ ']', "&cb;", 4 },
							{ '=', "&eq;", 4 },
							{ '"', "&dq;", 4 },
							{ '\'', "&sq;", 4 },
							{ '\"', "&quot;", 6 },
							{ '\xA0', "&nbsp;", 0 },
							{ 0, NULL, 0 } };

//static char* pszCodedS = "%&></?!-[]=\"'";
static const char* pszCodedS = "&><\xA0";

USE_RFC_NAMESPACE

const String XMLUtil::XMLDecode( const String& strEncoded )
{
/*
	Search for ampersand. When found, search for semi-colon. What is in between
	is the encoded character. Replace the sequence &[...]; with the appropriate
	character.
*/

	if( strEncoded.isNull() )
		return String::null;

	String strRtn;
	char szTag[16];
	String strTemp( strEncoded );
	char* pszCurrent = strTemp.buffer();
	char* pszStartTag;
	char* pszEndTag;
	unsigned long ulTagIndex;

	pszStartTag = strchr( pszCurrent, '&' );
	while( pszStartTag != NULL )
	{
		strncpy( szTag, pszStartTag, sizeof szTag );
		pszEndTag = strchr( szTag, ';' );
		if( pszEndTag == NULL )
			throw XMLException( XE_U_UNTERMINATEDCODE );

		// Isolate the tag in the tag buffer
		*( pszEndTag + 1 ) = ASC_NUL;
		*pszStartTag = ASC_NUL;
		// Copy in the text up to the tag
		strRtn += pszCurrent;

		if( szTag[1] == '#' )
		{
			char ch = (char)atoi( &szTag[2] );
			strRtn += ch;
		}
		else
		{
			// Go looking for the tag translation
			ulTagIndex = 0;
			while( ( map[ulTagIndex].pszCode != NULL ) && ( strcmp( map[ulTagIndex].pszCode, szTag ) ) )
				ulTagIndex++;
			if( map[ulTagIndex].pszCode == NULL )
				throw XMLException( XE_U_INVALIDCODE );

			strRtn += map[ulTagIndex].cChar;
		}

		pszCurrent = strchr( ( pszStartTag + 1 ), ';' ) + 1;
		pszStartTag = strchr( pszCurrent, '&' );
	}

	// Copy in the remainder
	strRtn += pszCurrent;

//	pszCurrent = strRtn.buffer();
//	pszStartTag = strchr( pszCurrent, '+' );
//	while( pszStartTag != NULL )
//	{
//		*pszStartTag = ASC_SPACE;
//		pszStartTag = strchr( pszCurrent, '+' );
//	}

#ifdef SLAP_JONA_FOR_THIS
	strTemp = strRtn;
	strRtn = "";
	pszCurrent = strTemp.buffer();
	pszStartTag = strchr( pszCurrent, '%' );
	while( pszStartTag != NULL )
	{
		strncpy( szTag, pszStartTag, 3 );
		szTag[3] = ASC_NUL;

		*pszStartTag = ASC_NUL;
		// Copy in the text up to the tag
		strRtn += pszCurrent;

		char ch = (char)strtol( &szTag[1], NULL, 16 );
		strRtn += ch;

		pszCurrent = ( pszStartTag + 3 );
		pszStartTag = strchr( pszCurrent, '%' );
	}

	// Copy in the remainder
	strRtn += pszCurrent;
#endif /* SLAP_JONA_FOR_THIS */

	return strRtn;
}

const String XMLUtil::XMLEncode( const String& strDecoded )
{
//strpbrk( s, accept )
/*
	Search for ampersand. When found, search for semi-colon. What is in between
	is the encoded character. Replace the sequence &[...]; with the appropriate
	character.
*/

	if( strDecoded.isNull() )
		return String::null;

	String strRtn;
	char cTag;
	String strTemp( strDecoded );
	char* pszCurrent = strTemp.buffer();
	char* pszStartTag;
	unsigned long ulTagIndex;

	pszStartTag = strpbrk( pszCurrent, pszCodedS );
	while( pszStartTag != NULL )
	{
		cTag = *pszStartTag;
		*pszStartTag = ASC_NUL;
		// Copy in the text up to the tag
		strRtn += pszCurrent;
		// Go looking for the tag translation
		ulTagIndex = 0;
		while( ( map[ulTagIndex].pszCode != NULL ) && ( map[ulTagIndex].cChar != cTag ) )
			ulTagIndex++;
		if( map[ulTagIndex].pszCode == NULL )
			throw XMLException( XE_U_INVALIDCODE );

		strRtn += map[ulTagIndex].pszCode;

		pszCurrent = pszStartTag + 1;
		pszStartTag = strpbrk( pszCurrent, pszCodedS );
	}

	// Copy in the remainder
	strRtn += pszCurrent;

	return strRtn;
}

const String& XMLUtil::getNode( const String& strSource, bool* pbEmpty )
{

	return String::null;
}
